"""
	操作sqlite3数据库
	Operating sqlite3 database
"""
import os,sys,time
import sqlite3
import traceback 
from .WriteReadFileFunc import  WriteReadFile

class Sqlite3DataBaseHandle(WriteReadFile):
	def init_sqlite3(self) -> bool:
		self.Path = f"{os.getcwd()}/ConfigFile.json"
		if not os.path.exists(self.Path):
			with open(self.Path,"w",encoding='UTF-8') as w:
				w.write('')

		Sqlite3_DB_Path = WriteReadFile.ReadJsonFile(self,self.Path,"Sqlite3_DB")

		# 打开数据库连接
		try:
			self.conn = sqlite3.connect(Sqlite3_DB_Path, check_same_thread=False)
			self.cursor = self.conn.cursor()
			return True
		except Exception as e:
			print("""请在ConfigFile.json 中配置你的数据库信息示例：
			 		{
						"Sqlite3_DB":"D:\\Project\\Coding\\PyAibote\\DB\\Pyaibote.db"
					}
				""")
			WriteReadFile.Custom_Write_logger(self,"",f"{os.getcwd()}/AiBotRunLOG/{time.strftime(r'%Y-%m-%d',time.localtime(time.time()))}/","SysModeError.log",False,"error",f"{traceback.format_exc()}")
			return False

	def dict_factory(self, cursor, row) -> list:
		"""
			元祖变成字典
			Yuanzu became a dictionary

			cursor: sql对象
			row: 查询出来的数据集
			return: 列表

			cursor: sql object
			row: the queried data set
			return: list
		"""
		DictList = []
		if row:
			col = cursor.description
			for RowData in row:
				DataDict = {}
				for idx  in range(len(RowData)):
					DataDict[col[idx][0]] = RowData[idx]
				DictList.append(DataDict)
		return DictList


	def insert_delete_update_sqlite3(self,sql) -> bool:
		"""
			新增, 删除, 更新 sqlite3数据库数据
			Insert data, delete data, update data

			sq: 插入数据, 删除数据, 更新数据的sql语句
			return: 成功返回True, 失败返回False

			sql:sql statements for inserting data, deleting data and updating data
			return:  True on success and False on failure
		"""
		try:
			self.cursor.execute(sql)
			self.conn.commit()
			return True
		except Exception as e:
			WriteReadFile.Custom_Write_logger(self,"",f"{os.getcwd()}/AiBotRunLOG/{time.strftime(r'%Y-%m-%d',time.localtime(time.time()))}/","SysModeError.log",False,"error",f"{traceback.format_exc()}")
			return False

	def select_sqlite3(self,sql)  -> list: 
		"""
			查询sqlite3数据库
			Query sqlite3 database

			sql: 查询数据库的sql语句
			return: 查询的列表数据数据或者False

			sql: sql statement for querying the database
			return: Query list data data or False
		"""
		try:
			cur = self.cursor.execute(sql)
			data = cur.fetchall()
			data = self.dict_factory(cur, data)
			return data
		except Exception as e:
			WriteReadFile.Custom_Write_logger(self,"",f"{os.getcwd()}/AiBotRunLOG/{time.strftime(r'%Y-%m-%d',time.localtime(time.time()))}/","SysModeError.log",False,"error",f"{traceback.format_exc()}")
			return False

	def close_sqlite3(self)  -> bool:
		"""
			关闭sqlite3数据库
			Close the sqlite3 database

			return: 布尔值
			return: boolean value
		"""
		try:
			self.conn.close()
			return True
		except Exception as e:
			WriteReadFile.Custom_Write_logger(self,"",f"{os.getcwd()}/AiBotRunLOG/{time.strftime(r'%Y-%m-%d',time.localtime(time.time()))}/","SysModeError.log",False,"error",f"{traceback.format_exc()}")
			return False






import re
import time

class ColorFindingOperation:
    """
        色值相关
        Color value correlation
    """

    def get_color(self, point: tuple) -> str:
        """
            获取指定坐标点的色值
            Gets the color value of the specified coordinate point.

            point: x, y坐标点
            return: 色值字符串(例如: #008577)或者 None

            point: x, y coordinate point
            return: color value string (for example: #008577) or None
        """
        response = self.SendData("getColor", point[0], point[1])
        if response == "null":
            return None
        return response

    def find_color(self, color: str, sub_colors: list = [], region: tuple = (0,0,0,0), similarity: float = 0.9, wait_time: float = 5, interval_time: float = 0.5) -> tuple:
        """
            获取指定色值的坐标点，返回坐标或者 None
            Gets the coordinate point of the specified color value, and returns the coordinate or None

            color: 颜色字符串，必须以 # 开头，例如：#008577；
            sub_colors: 辅助定位的其他颜色 (偏移x、偏移y、颜色字符串)
            region: 截图区域，默认全屏，``region = (起点x、起点y、终点x、终点y)``，得到一个矩形
            similarity: 相似度，0-1 的浮点数，默认 0.9；
            wait_time: 等待时间，默认取 5秒
            interval_time: 轮询间隔时间，默认取 0.5秒
            return: 坐标或者 ()

            color: color string, which must start with #, for example: # 008577
            sub_colors: other colors (offset x, offset y, color string) to assist positioning
            region: screenshot area, full screen by default, `` region = (starting point x, starting point y, ending point x, ending point y) ```, and a rectangle is obtained
            similarity: similarity, floating point number of 0-1, default 0.9
            wait_time: waiting time, which is 5 seconds by default
            interval_time: the polling interval, which is 0.5 seconds by default
            return: coordinates or ()
        """
        if sub_colors:
            sub_colors_str = ""
            for sub_color in sub_colors:
                offset_x, offset_y, color_str = sub_color
                sub_colors_str += f"{offset_x}/{offset_y}/{color_str}\n"
            sub_colors_str = sub_colors_str.strip()
        else:
            sub_colors_str = "null"
        end_time = time.time() + wait_time
        while time.time() < end_time:
            response = self.SendData("findColor", color, sub_colors_str, *region, similarity)
            if response == "-1.0|-1.0":
                time.sleep(interval_time)
            else:
                x, y = response.split("|")
                return (x, y)
        return ()








import re
import time,json


class ElementOperation:
    """
        元素操作
        Element operation
    """
    
    def get_element_rect(self, xpath: str, wait_time: float = 5, interval_time: float = 0.5) -> tuple:
        """
            获取元素位置，返回元素区域左上角和右下角坐标
            Gets the position of the element and returns the coordinates of the upper left corner and the lower right corner of the element area

            xpath: xpath 路径
            wait_time: 等待时间，默认取 5秒
            interval_time: 轮询间隔时间，默认取 0.5秒
            return: x,y坐标 或者 空()

            xpath: xpath path
            wait_time: waiting time, which is 5 seconds by default
            interval_time: the polling interval, which is 0.5 seconds by default
            return: x,y coordinates or empty ()
        """

        end_time = time.time() + wait_time
        while time.time() < end_time:
            response = self.SendData("getElementRect", xpath)
            if response == "-1|-1|-1|-1":
                time.sleep(interval_time)
            else:
                start_x, start_y, end_x, end_y = response.split("|")
                return ((float(end_x)+float(start_x))/2,(float(end_y)+float(start_y))/2)

        return ()

    def press_release_by_ele(self, xpath, duration: float, wait_time: float = 5, interval_time: float = 0.5) -> bool:
        """
            按压元素并释放
            Press the element and release.

            xpath: 要按压的元素
            duration: 按压时长，单位秒
            wait_time: 查找元素的最长等待时间 默认5秒
            interval_time: 查找元素的轮询间隔时间 默认0.5
            return: Ture 或者 False

            xpath: the element to press
            duration: duration of pressing, in seconds
            wait_time: The longest waiting time for finding an element defaults to 5 seconds
            interval_time: The polling interval of the lookup element defaults to 0.5
            return: Ture or False
        """
        response = self.get_element_rect(xpath, wait_time=wait_time, interval_time=interval_time)
        if response:
            return self.press_release(response, duration)
        else:
            return False

    def get_element_desc(self, xpath: str, wait_time: float = 5, interval_time: float = 0.5) -> str:
        """
            获取元素描述
            Get element description

            xpath: xpath 路径
            wait_time: 等待时间，默认取 5秒
            interval_time: 轮询间隔时间，默认取 
            return: 元素描述字符串

            xpath: xpath path
            wait_time: the waiting time, which is self.wait_timeout by default to 5 seconds
            interval_time: polling interval; self.interval_timeout is selected by default to 0.5
            return: element description string 
        """

        end_time = time.time() + wait_time
        while time.time() < end_time:
            response = self.SendData("getElementDescription", xpath)
            if response == "null":
                time.sleep(interval_time)
            else:
                return response
        return None

    def get_element_text(self, xpath: str, wait_time: float = 5, interval_time: float = 0.5) -> str:
        """
            获取元素文本
            Get element text

            xpath: xpath 路径
            wait_time: 等待时间，默认取 5秒
            interval_time: 轮询间隔时间，默认取 0.5秒
            return: 元素文本

            xpath: xpath path
            wait_time: waiting time, which is 5 seconds by default.
            interval_time: the polling interval, which is 0.5 seconds by default.
            return: element text
        """

        end_time = time.time() + wait_time
        while time.time() < end_time:
            response = self.SendData("getElementText", xpath)
            if response == "null":
                time.sleep(interval_time)
            else:
                return response
        return None

    def set_element_text(self, xpath: str, text: str, wait_time: float = 5, interval_time: float = 0.5) -> bool:
        """
            设置元素文本
            Set element text

            xpath: 元素路径
            text: 设置的文本
            wait_time: 等待时间，默认取 self.wait_timeout
            interval_time: 轮询间隔时间，默认取 self.interval_timeout
            return:  成功返回True 失败返回False

            xpath: element path
            text: the text of the setting
            wait_time: the waiting time, which is self.wait_timeout by default
            interval_time: polling interval; self.interval_timeout is selected by default
            return: Returns True successfully, and returns False if it fails
        """
  
        end_time = time.time() + wait_time
        while time.time() < end_time:
            response = self.SendData("setElementText", xpath, text)
            if "false" in response or "null" in response:
                time.sleep(interval_time)
            else:
                return True
        return False

    def click_element(self, xpath: str, wait_time: float = 5, interval_time: float = 0.5) -> bool:
        """
            点击元素
            Click element

            xpath: 元素路径
            wait_time: 等待时间，默认取 5秒
            interval_time: 轮询间隔时间，默认取 0.5秒
            return: 成功返回True 失败返回False

            xpath: element path
            wait_time: waiting time, which is 5 seconds by default
            interval_time: the polling interval, which is 0.5 seconds by default
            return: Returns True successfully, and returns False if it fails
        """

        end_time = time.time() + wait_time
        while time.time() < end_time:
            response = self.SendData("clickElement", xpath)
            if "false" in response or "null" in response:
                time.sleep(interval_time)
            else:
                return True
        return False

    def click_any_elements(self, xpath_list: list, wait_time: float = 5, interval_time: float = 0.5) -> bool:
        """
            遍历点击列表中的元素，直到任意一个元素返回 True
            Traverse the elements in the click list until any element returns True

            xpath_list: xpath 列表
            wait_time: 等待时间，默认取 5秒
            interval_time: 轮询间隔时间，默认取 0.5秒
            return: 成功返回True 失败返回False

            xpath_list: xpath list
            wait_time: waiting time, which is 5 seconds by default
            interval_time: the polling interval, which is 0.5 seconds by default
            return: Returns True successfully, and returns False if it fails
        """

        end_time = time.time() + wait_time
        while time.time() < end_time:
            for xpath in xpath_list:
                result = self.click_element(xpath, wait_time, interval_time)
                if result:
                    return True
            time.sleep(interval_time)

        return False

    def scroll_element(self, xpath: str, direction: int = 0) -> bool:
        """
            滚动元素，0 向上滑动，1 向下滑动

            xpath: xpath 路径
            direction: 滚动方向，0 向上滑动，1 向下滑动
            return: 成功返回True 失败返回False
        """
        return "true" in self.SendData("scrollElement", xpath, direction) 

    def element_exists(self, xpath: str, wait_time: float = 5, interval_time: float = 0.5) -> bool:
        """
            元素是否存在
            Does the element exist

            xpath: xpath 路径
            wait_time: 等待时间，默认取 self.wait_timeout
            interval_time: 轮询间隔时间，默认取 self.interval_timeout
            return: 成功返回True 失败返回False

            xpath: xpath path
            wait_time: the waiting time, which is self.wait_timeout by default
            interval_time: polling interval; self.interval_timeout is selected by default
            return: Returns True successfully, and returns False if it fails
        """
        end_time = time.time() + wait_time
        while time.time() < end_time:
            response = self.SendData("existsElement", xpath) 
            if "false" in response or "null" in response:
                time.sleep(interval_time)
            else:
                return True
        return False

    def any_elements_exists(self, xpath_list: list, wait_time: float = 5, interval_time: float = 0.5) -> bool:
        """
            遍历列表中的元素
            Traverse the elements in the list

            xpath_list: xpath 列表
            wait_time: 等待时间，默认取 self.wait_timeout
            interval_time: 轮询间隔时间，默认取 self.interval_timeout
            return: 任意一个元素存在就返回 True 否则就是为False

            xpath_list: xpath list
            wait_time: the waiting time, which is self.wait_timeout by default
            interval_time: polling interval; self.interval_timeout is selected by default
            return: If any element exists, it will return True, otherwise it will be False
        """

        end_time = time.time() + wait_time
        while time.time() < end_time:
            for xpath in xpath_list:
                result = self.element_exists(xpath, wait_time, interval_time)
                if result:
                    return result
            time.sleep(interval_time)
        return result

    def element_is_selected(self, xpath: str) -> bool:
        """
            判断元素是否选中
            Determine whether the element is selected

            xpath: xpath 路径
            return: 成功返回True 失败返回False

            xpath: xpath path
            return: Returns True successfully, and returns False if it fails
        """
        return  "true" in self.SendData("isSelectedElement", xpath) 

    def click_element_by_slide(self, xpath, distance: int = 1000, duration: float = 0.5, direction: int = 1,count: int = 999, end_flag_xpath: str = None, wait_time: float = 600,interval_time: float = 0.5) -> bool:
        """
            滑动列表，查找并点击指定元素
            Slide the list, find and click the specified element.

            xpath: xpath路径
            distance: 滑动距离，默认 1000
            duration: 滑动时间，默认 0.5 秒
            direction: 滑动方向，默认为 1； 1=上滑，2=下滑
            count: 滑动次数
            end_flag_xpath: 结束标志 xpath，无标志不检测此标志
            wait_time: 等待时间，默认 10 分钟
            interval_time: 轮询间隔时间，默认 0.5 秒
            return: 成功返回True 失败返回False

            Xpath: xpath path
            Distance: sliding distance; default is 1000
            Duration: sliding time, 0.5 seconds by default
            Direction: sliding direction, which defaults to 1; 1= slide up, 2= slide down
            Count: number of slides
            End_flag_xpath: end the flag xpath, if there is no flag, this flag will not be detected
            Wait_time: waiting time, 10 minutes by default
            Interval_time: the polling interval, which is 0.5 seconds by default
            return:  Returns True successfully, and returns False if it fails
        """

        if direction == 1:
            _end_point = (500, 300)
            _start_point = (500, _end_point[1] + distance)
        elif direction == 2:
            _start_point = (500, 300)
            _end_point = (500, _start_point[1] + distance)

        end_time = time.time() + wait_time
        current_count = 0
        while time.time() < end_time and current_count < count:
            current_count += 1

            if self.click_element(xpath, wait_time=1, interval_time=0.5):
                return True

            if end_flag_xpath and self.element_exists(end_flag_xpath, wait_time=1, interval_time=0.5):
                return False

            self.swipe(_start_point, _end_point, duration)
            time.sleep(interval_time)

        return False

    def get_elements(self) -> json:
        """
            获取可见区域内的所有元素信息
            return: 成功返回数组json格式的元素信息，失败返回null

            Get information of all elements in the visible area
            return: element information in json format of array is returned successfully, and null is returned if it fails
        """
        response = self.SendData("getElements")
        if response == "null":
            return None
        return json.loads(response)


"""
    读写文件
    WriteReadFile
"""

from .LoggerFunc import LoggerRecord
import os,json,time
import traceback

class WriteReadFile(LoggerRecord):
    def WriteFile(self, path, data) -> bool:
        """
            写入文件
            WriteFile

            path: 需要写入的文件路径
            data: 需要写入的数据
            return: 成功返回True or 失败返回False

            path: the file path that needs to be written
            data: data to be written
            return: Returns True or success or False on failure
        """

        try:
            with open(path,'w',encoding='UTF-8') as w:
                w.write(data)
            return True
        except Exception as e:
            WriteReadFile.Custom_Write_logger(self,"",f"{os.getcwd()}/AiBotRunLOG/{time.strftime(r'%Y-%m-%d',time.localtime(time.time()))}/","SysModeError.log",False,"error",f"{traceback.format_exc()}")
            return False

    def ReadFile(self,path) -> str or bool:
        """
            读取文件
            ReadFile

            path: 需要读取的文件路径
            return: 成功返回文件数据, 失败返回False

            path: the file path to be read
            return: file data is returned successfully, and False is returned if it fails
        """

        try:
            with open(path,'r',encoding='UTF-8') as r:
                Data = r.read()
            return Data
        except Exception as e:
            WriteReadFile.Custom_Write_logger(self,"",f"{os.getcwd()}/AiBotRunLOG/{time.strftime(r'%Y-%m-%d',time.localtime(time.time()))}/","SysModeError.log",False,"error",f"{traceback.format_exc()}")
            return False

    def WriteFileAddTo(self,path,data)  -> bool:
        """
            追加写入文件
            Additional write file

            path: 需要追加写入的文件路径
            data: 需要追加写入的数据
            return: 成功返回True, 失败返回False

            path: the file path that needs additional writing
            data: data that needs to be written additionally
            return: file data is returned successfully, and False is returned if it fails
        """

        try:
            with open(path,'a',encoding='UTF-8') as w:
                w.write(data)
            return True
        except Exception as e:
            WriteReadFile.Custom_Write_logger(self,"",f"{os.getcwd()}/AiBotRunLOG/{time.strftime(r'%Y-%m-%d',time.localtime(time.time()))}/","SysModeError.log",False,"error",f"{traceback.format_exc()}")
            return False

    def ReadJsonFile(self, path, key=None, Key2=None, Key3=None, Key4=None, Key5=None) -> str or bool:
        """
            读取json文件
            Read json file

            key: 读取一级嵌套路径信息
            Key2: 读取key二级嵌套路径信息
            ....  最多支持五级嵌套路径, 默认读取所有json文件内的信息
            return: 成功返回json文件字典数据, 失败返回False
            

            Key: Read the first-level nested path information
            Key2: read the second-level nested path information of key
            ... supports up to five nested paths, and reads the information in all json files by default
            return: json file data is returned successfully, and False is returned if it fails
        """

        try:
            ResultData = self.ReadFile(path)
            ResultData = json.loads(ResultData)
            if key and Key2 and Key3 and Key4 and Key5:
                ResultData = ResultData[key][Key2][Key3][Key4][Key5]
            elif key and Key2 and Key3 and Key4:
                ResultData = ResultData[key][Key2][Key3][Key4]
            elif key and Key2 and Key3:
                ResultData = ResultData[key][Key2][Key3]
            elif key and Key2:
                ResultData = ResultData[key][Key2]
            elif key:
                ResultData = ResultData[key]
            return ResultData
        except Exception as e:
            WriteReadFile.Custom_Write_logger(self,"",f"{os.getcwd()}/AiBotRunLOG/{time.strftime(r'%Y-%m-%d',time.localtime(time.time()))}/","SysModeError.log",False,"error",f"{traceback.format_exc()}")
            return False














import re
import time


class WindowOperation:
    """
        窗口操作
        Window operation
    """

    def find_window(self, class_name: str = None, window_name: str = None) -> str:
        """
            查找窗口句柄，仅查找顶级窗口，不包含子窗口
            Find the window handle, only the top-level window, not the child window

            class_name: 窗口类名
            window_name: 窗口名
            return: 字符串

            class_name: Window class name
            window_name: Window name
            return: character string
        """
        response = self.SendData("findWindow", class_name, window_name)
        if response == "null":
            return None
        return response

    def find_windows(self, class_name: str = None, window_name: str = None) -> str:
        """
            查找窗口句柄数组，仅查找顶级窗口，不包含子窗口 class_name 和 window_name 都为 None，则返回所有窗口句柄
            Find an array of window handles, only top-level windows are found, and all window handles are returned if class_name and window_name are None

            class_name: 窗口类名
            window_name: 窗口名
            return: 窗口句柄的列表

            class_name: Window class name
            window_name: Window name
            return: List of window handles
        """
        response = self.SendData("findWindows", class_name, window_name)
        if response == "null":
            return []
        return response.split("|")

    def find_sub_window(self, hwnd: str, class_name: str = None, window_name: str = None) -> str:
        """
            查找子窗口句柄
            Find child window handle

            hwnd: 当前窗口句柄
            class_name: 窗口类名
            window_name: 窗口名
            return: 子窗口句柄或 None

            hwnd: Current window handle
            class_name: Window class name
            window_name: Window name
            return: Child window handle or None
        """
        response = self.SendData("findSubWindow", hwnd, class_name, window_name)
        if response == "null":
            return None
        return response

    def find_parent_window(self, hwnd: str) -> str:
        """
            查找父窗口句柄
            Find the parent window handle

            hwnd: 当前窗口句柄
            return: 父窗口句柄或 None

            hwnd: Current window handle
            return: Parent window handle or None
        """
        response = self.SendData("findParentWindow", hwnd)
        if response == "null":
            return None
        return response

    def find_desktop_window(self) -> str:
        """
            查找桌面窗口句柄
            Find the desktop window handle

            return: 桌面窗口句柄或None
            return: Desktop window handle or None
        """
        response = self.SendData("findDesktopWindow")
        if response == "null":
            return None
        return response

    def get_window_name(self, hwnd: str) -> str:
        """
            获取窗口名称
            Get the window name

            hwnd: 当前窗口句柄
            return: 窗口名称或 None

            hwnd: Current window handle
            return: Window name or None
        """
        response = self.SendData("getWindowName", hwnd)
        if response == "null":
            return None
        return response

    def show_window(self, hwnd: str, isShow: bool) -> bool:
        """
            显示/隐藏窗口
            Show/hide window

            hwnd: 当前窗口句柄
            isShow: 显示窗口 True 隐藏窗口 False
            return: 布尔值

            hwnd: Current window handle
            isShow: Show window True hide window False
            return: bool
        """
        return "true" in self.SendData("showWindow", hwnd, isShow)

    def get_window_pos(self, hwnd: str, wait_time: float = 5, interval_time: float = 0.5) -> dict:
        """
            获取窗口大小和位置
            Get the window size and location

            hwnd: 窗口句柄
            wait_time: 循环等待的总时间 默认5秒
            interval_time: 每隔多少时间重试一次 默认0.5
            return: left和top：windwos窗口最左侧为起点坐标，width和height：句柄窗口大小，或者超时返回 None

            hwnd: Window handle
            wait_time: Total time of loop waiting
            interval_time: How often do you try again
            return: The leftmost side of the windwos window is the starting point coordinates, width and height: handle window size, or return None
        """

        end_time = time.time() + wait_time
        while time.time() < end_time:
            response = self.SendData("getWindowPos", hwnd)
            if response == "-1|-1|-1|-1":
                time.sleep(interval_time)
                continue
            else:
                x1, y1, x2, y2 = response.split("|")
                return {"left": x1, "top": y1, "width": x2, "height": y2}
        return None

    def set_window_pos(self, hwnd: str, left: float, top: float, width: float, height: float) -> bool:
        """
            设置窗口位置
            Set window position

            hwnd: 当前窗口句柄
            left: 左上角横坐标
            top: 左上角纵坐标
            width: 窗口宽度
            height: 窗口高度
            return: 布尔值

            hwnd: Current window handle
            left: Upper left abscissa
            top: Upper left ordinate
            width: Window width
            height: Window height
            return: bool
        """
        return "true" in self.SendData("setWindowPos", hwnd, left, top, width, height)

    def set_window_top(self, hwnd: str, top: bool) -> bool:
        """
            设置窗口到最顶层
            Set the window to the top level

            hwnd: 当前窗口句柄
            top: 是否置顶，True 置顶， False 取消置顶
            return: 布尔值

            hwnd: Current window handle
            top: Set top or not, True set top, False cancel set top
            return: bool
        """
        return "true" in self.SendData("setWindowTop", hwnd, top)

    def coordinate_transform(self, coordinate: tuple, resolution_a: tuple, resolution_b: tuple) -> tuple:
        """
            根据屏幕分辨率缩放比例，进行不同分辨率坐标转换
            According to the screen resolution scaling, coordinate conversion with different resolutions is carried out.

            coordinate: 需要装换的坐标可以是x,y坐标，也可以是矩形坐标 x1, y1, x2, y2
            resolution_a: 你抓取过坐标的设备标准分辨率
            resolution_b：你需要转换的设备分辨率
            retrun: 元祖

            coordinate: the coordinates to be replaced can be x,y coordinates or rectangular coordinates x1, y1, x2, y2
            resolution_a: The standard resolution of the device for which you have grabbed the coordinates
            resolution_b: The device resolution you need to convert
            retrun: Yuanzu
        """
        
        def convert_coordinates(x, y, resolution_a, resolution_b):
            W_A, H_A = resolution_a  
            W_B, H_B = resolution_b 
            

            scale_x = W_B / W_A
            scale_y = H_B / H_A
            

            x_b = x * scale_x
            y_b = y * scale_y
            
            return (x_b, y_b)

        if len(coordinate) == 2:
            result = convert_coordinates(coordinate[0], coordinate[1], resolution_a, resolution_b)
            return result

        if len(coordinate) == 4:
            result = convert_coordinates(coordinate[0], coordinate[1], resolution_a, resolution_b)
            result2 = convert_coordinates(coordinate[2], coordinate[3], resolution_a, resolution_b)
            return (result[0], result[1], result2[0], result2[1])
        return ()


import os,re

class DrivingOperation:
    """
        驱动操作
        Driving operation
    """

    def close_driver(self) -> bool:
        """
            远程部署时关闭WindowsDriver.exe驱动程序
            Close the WindowsDriver.exe driver

            return: 布尔值
            return: bool
        """

        return "true" in self.SendData("closeDriver")

    def close_driver_local(self) -> bool:
        """
            本地部署时关闭WindowsDriver.exe驱动程序(避免堆积大量WindowsDriver.exe进程占用系统资源)
            Close the WindowsDriver.exe driver

            return: 布尔值
            return: bool
        """

        os.system('taskkill /f /t /im  "WindowsDriver.exe"')
        return True 

    def activate_frame(self, secret_key) -> str:
        """
            激活Windows框架
            Activate Windows framework

            secret_key：激活框架的秘钥
            secret_key：Key to activate the frame

            return: 成功返回True失败返回False
            return: Returns True successfully and False if it fails
        """

        response = self.SendData("activateFrame", secret_key) 
        return response

    def get_extend_param(self) -> str:
        """
            获取驱动程序命令行参数(不包含ip和port)
            Get the driver command line parameters (excluding ip and port)

            return: 成功返回参数，失败返回None
            return: parameter is returned successfully, and None is returned on failure.
        """

        response = self.SendData("getExtendParam")
        if response == "null":
            return None
        return response

    def get_windows_id(self) -> str:
        """
            获取Windows 唯一ID用于区分机器
            Get Windows unique ID to distinguish machines.

            return: 成功返回参数，失败返回None
            return: Parameter is returned successfully, and None is returned on failure.
        """

        response = self.SendData("getWindowsId")
        if response == "null":
            return None
        return response


import re
import time

class MapFindingOperation:
    """
        找图功能
        Map finding function
    """
    def find_images(self, image_name: str, region: tuple = (0,0,0,0), algorithm: tuple = (0,0,0), similarity: float = 0.9, multi: int = 1, wait_time: float = 5, interval_time: float = 0.5) -> list:
        """
            寻找图片坐标，在当前屏幕中寻找给定图片中心点的坐标，返回坐标列表(如果想拿一个坐标取第一个坐标元祖即可)
            Find the coordinates of the picture, find the coordinates of the center point of a given picture in the current screen, and return to the coordinate list (if you want to take a coordinate and take the first coordinate ancestor)

            image_name: 图片名称（手机中）/storage/emulated/0/ 手机根目录
            region: 截图区域，默认全屏，``region = (起点x、起点y、终点x、终点y)``，得到一个矩形
            algorithm:
                处理屏幕截图所用的算法，默认原图，注意：给定图片处理时所用的算法，应该和此方法的算法一致；
                ``algorithm = (algorithm_type, threshold, max_val)``

                按元素顺序分别代表：
                0. ``algorithm_type`` 算法类型
                1. ``threshold`` 阈值
                2. ``max_val`` 最大值

                ``threshold`` 和 ``max_val`` 同为 255 时灰度处理.
                ``algorithm_type`` 算法类型说明:
                    0. ``THRESH_BINARY``      算法，当前点值大于阈值 `threshold` 时，取最大值 ``max_val``，否则设置为 0；
                    1. ``THRESH_BINARY_INV``  算法，当前点值大于阈值 `threshold` 时，设置为 0，否则设置为最大值 max_val；
                    2. ``THRESH_TOZERO``      算法，当前点值大于阈值 `threshold` 时，不改变，否则设置为 0；
                    3. ``THRESH_TOZERO_INV``  算法，当前点值大于阈值 ``threshold`` 时，设置为 0，否则不改变；
                    4. ``THRESH_TRUNC``       算法，当前点值大于阈值 ``threshold`` 时，设置为阈值 ``threshold``，否则不改变；
                    5. ``ADAPTIVE_THRESH_MEAN_C``      算法，自适应阈值；
                    6. ``ADAPTIVE_THRESH_GAUSSIAN_C``  算法，自适应阈值；
            similarity: 相似度，0-1 的浮点数，默认 0.9；
            multi: 目标数量，默认为 1，找到 1 个目标后立即结束；
            wait_time: 等待时间，默认取 self.wait_timeout
            interval_time: 轮询间隔时间，默认取 self.interval_timeout
            return: 坐标列表 或者 []

            image_name: image name (in mobile phone) /storage/emulated/0/ mobile phone root directory.
            region: screenshot area, full screen by default, `` region = (starting point x, starting point y, ending point x, ending point y) ```, and a rectangle is obtained.
            algorithm:
                The algorithm used to process screen shots defaults to the original image. Note: the algorithm used in the given image processing should be consistent with the algorithm of this method;

                ``algorithm = (algorithm_type, threshold, max_val)``

                Represents, respectively, in the order of elements:
                    0. ``algorithm_type ``` algorithm type
                    1. ``threshold ```` threshold
                    2. `` Max _ val```` Maximum value

                Gray processing when threshold and max_val are both 255
                `` algorithm _ type``` description of algorithm_type
                        0. ``` threshold _ binary``` algorithm, when the current point value is greater than the threshold value ` threshold `, the maximum value ``` max _ val```` is taken, otherwise it is set to 0
                        1. `` threshold _ binary _ inv``` algorithm, when the current point value is greater than the threshold value ` threshold `, set it to 0, otherwise set it to the maximum value max_val；
                        2. `` threshold _ tozero``` algorithm, when the current point value is greater than the threshold ` threshold', it will not be changed, otherwise it will be set to 0
                        3. `` threshold _ tozero _ inv``` algorithm, when the current point value is greater than the threshold value ``` threshold ```, it is set to 0, otherwise it will not change
                        4. `` threshold _ trunc``` algorithm, when the current point value is greater than the threshold value ``` threshold ```, it is set as the threshold value ``` threshold ```, otherwise it will not be changed
                        5. ``` Adaptive _ threshold _ mean _ c``` algorithm, adaptive threshold
                        6. ``` Adaptive _ Thresh _ Gaussian _ C``` algorithm, adaptive threshold
            similarity: similarity, floating point number of 0-1, default 0.9;
            multi: number of targets; the default value is 1, and it ends immediately after finding one target
            wait_time: the waiting time, which is self.wait_timeout by default
            interval_time: polling interval; self.interval_timeout is selected by default
            Return: list of coordinates or []
        """
        if not image_name.startswith("/storage/emulated/0/"):
            image_name = "/storage/emulated/0/" + image_name
    
        algorithm_type, threshold, max_val = algorithm
        if algorithm_type in (5, 6):
            threshold = 127
            max_val = 255
        end_time = time.time() + wait_time
        while time.time() < end_time:
            response = self.SendData("findImage", image_name, *region, similarity, algorithm_type, threshold, max_val, multi)
            if response == "-1.0|-1.0":
                time.sleep(interval_time)
            else:
                image_points = response.split("/")
                point_list = []
                for point_str in image_points:
                    x, y = point_str.split("|")
                    point_list.append((float(x),float(y)))
                return point_list
        return []

    def find_dynamic_image(self, interval_ti: int, region: tuple = (0,0,0,0), wait_time: float = 5, interval_time: float = 0.5) -> list:
        """
            找动态图，对比同一张图在不同时刻是否发生变化，返回坐标列表
            Find the dynamic diagram, compare whether the same diagram has changed at different times, and return the coordinate list

            interval_ti: 前后时刻的间隔时间，单位毫秒；
            region: 截图区域，默认全屏，``region = (起点x、起点y、终点x、终点y)``，得到一个矩形
            wait_time: 等待时间，默认取 5秒
            interval_time: 轮询间隔时间，默认取 0.5秒
            return: 坐标列表 或者 []

            interval_ti: the interval between the preceding and the following moments, in milliseconds
            region: screenshot area, full screen by default, `` region = (starting point x, starting point y, ending point x, ending point y) ```, and a rectangle is obtained
            wait_time: waiting time, which is 5 seconds by default
            interval_time: the polling interval, which is 0.5 seconds by default
            return: list of coordinates or []
        """
        end_time = time.time() + wait_time
        while time.time() < end_time:
            response = self.SendData("findAnimation", interval_ti, *region)
            if response == "-1.0|-1.0":
                time.sleep(interval_time)
            else:
                image_points = response.split("/")
                point_list = []
                for point_str in image_points:
                    x, y = point_str.split("|")
                    point_list.append((float(x), float(y)))
                return point_list
        return []









import re


class OcrCorrelation:
    """
        OCR 文字识别
        optical character recognition
    """
    def _parsing_ocr_data(self, ocr_server_id: str = "127.0.0.1", region: tuple = (0,0,0,0), algorithm: tuple = (0,0,0), scale: float = 1.0)-> list:
        """
            OCR 文字服务，识别屏幕中的文字
            OCR text service, which recognizes the text on the screen

            ocr_server_id: ocr服务端IP，端口固定为9527。当参数值为 "127.0.0.1"时，则使用手机内置的ocr识别
            region 指定区域 [10, 20, 100, 200]，region默认全屏
            algorithm: threshold二值化图片 (algorithm_type: 算法类型, threshold: 默认保存原图。thresh和maxval同为255时灰度处理 , max_val: 最大值)
                                                0   THRESH_BINARY算法，当前点值大于阈值thresh时，取最大值maxva，否则设置为0
                                                1   THRESH_BINARY_INV算法，当前点值大于阈值thresh时，设置为0，否则设置为最大值maxva
                                                2   THRESH_TOZERO算法，当前点值大于阈值thresh时，不改变，否则设置为0
                                                3   THRESH_TOZERO_INV 算法，当前点值大于阈值thresh时，设置为0，否则不改变
                                                4   THRESH_TRUNC算法，当前点值大于阈值thresh时，设置为阈值thresh，否则不改变
                                                5   ADAPTIVE_THRESH_MEAN_C算法，自适应阈值
                                                6   ADAPTIVE_THRESH_GAUSSIAN_C 算法，自适应阈值
            scale: 图片缩放率, 默认为 1.0 原大小。大于1.0放大，小于1.0缩小，不能为负数。
            return:  失败返回None，成功返回手机屏幕上的文字

            ocr_server_id: the IP of the ocr server, and the port is fixed at 9527. When the parameter value is "127.0.0.1", the built-in ocr of the mobile phone is used for recognition.
            region specifies the area [10, 20, 100, 200], and the region defaults to full screen
            algorithm: threshold binary image (algorithm_type: algorithm type, threshold: the original image is saved by default. Gray processing when thresh and maxval are both 255, max_val: maximum)
                0 THRESH_BINARY algorithm, when the current point value is greater than the threshold thresh, take the maximum maxva, otherwise set to 0
                1 THRESH_BINARY_INV algorithm, when the current point value is greater than the threshold thresh, set it to 0, otherwise set it to the maximum maxva
                2 THRESH_TOZERO algorithm, when the current point value is greater than the threshold thresh, it will not change, otherwise it will be set to 0
                3 THRESH_TOZERO_INV algorithm, when the current point value is greater than the threshold thresh, it is set to 0, otherwise it will not change
                4 THRESH_TRUNC algorithm, when the current point value is greater than the threshold thresh, set it as the threshold thresh, otherwise it will not change
                5 ADAPTIVE_THRESH_MEAN_C algorithm, adaptive threshold
                6 ADAPTIVE_THRESH_GAUSSIAN_C algorithm, adaptive threshold
            scale: the image scaling rate, which defaults to 1.0 original size. Magnification greater than 1.0 and reduction less than 1.0 cannot be negative
            return: return to None in case of failure, and successfully return the text on the mobile phone screen

        """

        algorithm_type, threshold, max_val = algorithm
        if algorithm_type in (5, 6):
            threshold = 127
            max_val = 255
        response = self.SendData("ocr", ocr_server_id, *region, algorithm_type, threshold, max_val, scale)
        if response == "null" or response == "":
            return []
        return eval(response)

    # def init_ocr_server(self, ip: str="127.0.0.1", useAngleModel:bool = False, enableGPU:bool =False, enableTensorrt:bool=False) -> bool:
    #     """
    #         初始化 OCR 服务
    #         Initialize OCR service
            
    #         ip: ocr服务IP或域名。端口固定 9527。当参数值为 "127.0.0.1"时，则使用手机内置的ocr识别，不必打开ppocr.exe服务端 默认值为 "127.0.0.1
    #         options: 可选参数，{useAngleModel:boolean, enableGPU:boolean, enableTensorrt:boolean}  仅外部OCR ocrServer ppocr.exe有效
    #             useAngleModel:   支持图像旋转。 默认False
    #             enableGPU:       启动GPU 模式。默认False 。GPU模式需要电脑安装NVIDIA驱动，并且到群文件下载对应cuda版本
    #             enableTensorrt : 启动加速，仅 enableGPU = true 时有效，默认False 。图片太大可能会导致GPU内存不足
    #         return: True 或者 False

    #         ip: ocr service IP or domain name. Port fixed 9527. When the parameter value is "127.0.0.1", the built-in ocr recognition of the mobile phone is used, and the default value is "127.0.0.1" without opening the ppocr.exe server
    #         options: optional parameter, {useanglemodel: Boolean, EnableGPU: Boolean, EnableTensort: Boolean} Only external OCR OCR Server ppocr.exe is valid.
    #             useAngleModel: supports image rotation. Default False
    #             enableGPU: start GPU mode. Default is False. GPU mode requires the computer to install NVIDIA driver and download the corresponding cuda version from the group file
    #             enableTensorrt: Starts acceleration, which is valid only when enableGPU = true, and the default is False. Too large a picture may lead to insufficient GPU memory
    #         return: True or False
    #     """
    #     return "true" in self.SendData("initOcr", ip,useAngleModel, enableGPU, enableTensorrt) 

    def get_text(self, ocr_server_id: str = "127.0.0.1", region: tuple = (0,0,0,0), algorithm: tuple = (0,0,0), scale: float = 1.0) -> list:
        """
            通过 OCR 识别屏幕中的文字，返回文字列表

            ocr_server_id: ocr服务端IP，端口固定为9527。当参数值为 "127.0.0.1"时，则使用手机内置的ocr识别
            region: 识别区域，默认全屏；
            algorithm: 处理图片/屏幕所用算法和参数，默认保存原图；
            threshold二值化图片, thresholdType算法类型：
                                                      0   THRESH_BINARY算法，当前点值大于阈值thresh时，取最大值maxva，否则设置为0
                                                      1   THRESH_BINARY_INV算法，当前点值大于阈值thresh时，设置为0，否则设置为最大值maxva
                                                      2   THRESH_TOZERO算法，当前点值大于阈值thresh时，不改变，否则设置为0
                                                      3   THRESH_TOZERO_INV算法，当前点值大于阈值thresh时，设置为0，否则不改变
                                                      4   THRESH_TRUNC算法，当前点值大于阈值thresh时，设置为阈值thresh，否则不改变
                                                      5   ADAPTIVE_THRESH_MEAN_C算法，自适应阈值
                                                      6   ADAPTIVE_THRESH_GAUSSIAN_C算法，自适应阈值
                                thresh阈值，maxval最大值，threshold默认保存原图。thresh和maxval同为255时灰度处理
            scale: 图片缩放率，默认为 1.0，1.0 以下为缩小，1.0 以上为放大
            return: 文字列表

            ocr_server_id: the IP of the ocr server, and the port is fixed at 9527. When the parameter value is "127.0.0.1", the built-in ocr of the mobile phone is used for recognition.
            region: recognition region, full screen by default;
            algorithm: the algorithm and parameters used to process the picture/screen, and the original picture is saved by default
            threshold binary image, thresholdType algorithm type:
                0 THRESH_BINARY algorithm, when the current point value is greater than the threshold thresh, take the maximum maxva, otherwise set to 0.
                1 THRESH_BINARY_INV algorithm, when the current point value is greater than the threshold thresh, set it to 0, otherwise set it to the maximum maxva.
                2 THRESH_TOZERO algorithm, when the current point value is greater than the threshold thresh, it will not change, otherwise it will be set to 0.
                3 THRESH_TOZERO_INV algorithm, when the current point value is greater than the threshold thresh, it is set to 0, otherwise it will not change.
                4 THRESH_TRUNC algorithm, when the current point value is greater than the threshold thresh, set it as the threshold thresh, otherwise it will not change.
                5 ADAPTIVE_THRESH_MEAN_C algorithm, adaptive threshold
                6 adaptive _ threshold _ Gaussian _ c algorithm, adaptive threshold.
            scale: the zoom rate of the picture, which is 1.0 by default; below 1.0, it means zooming out; above 1.0, it means zooming in
            return: text list
        """
        
        text_info_list = self._parsing_ocr_data(ocr_server_id, region, algorithm, scale)
        return text_info_list

    def find_text(self, ocr_server_id: str, text: str, region: tuple = (0,0,0,0), algorithm: tuple = (0,0,0), scale: float = 1.0) -> tuple:
        """
            查找文字所在的坐标，返回坐标列表（坐标是文本区域中心位置）
            Find the coordinates of the text and return a list of coordinates (coordinates are the center of the text area)

            ocr_server_id: ocr服务端IP，端口固定为9527。当参数值为 "127.0.0.1"时，则使用手机内置的ocr识别
            text: 要查找的文字；
            region: 识别区域，默认全屏；
            algorithm: 处理图片/屏幕所用算法和参数，默认保存原图；
            scale: 图片缩放率，默认为 1.0，1.0 以下为缩小，1.0 以上为放大；
            return: 坐标列表（坐标是文本区域中心位置）,找不到返回空列表 []

            ocr_server_id: the IP of the ocr server, and the port is fixed at 9527. When the parameter value is "127.0.0.1", the built-in ocr of the mobile phone is used for recognition.
            text: the text to find
            region: recognition region, full screen by default
            algorithm: the algorithm and parameters used to process the picture/screen, and the original picture is saved by default
            scale: the zoom rate of the picture, which is 1.0 by default; below 1.0, it is reduced; above 1.0, it is enlarged
            return: list of coordinates (coordinates are the center position of the text area), and the return empty list [] cannot be found
        """

        text_info_list = self._parsing_ocr_data(ocr_server_id, region, algorithm, scale)
        if text_info_list ==[]:
            return text_info_list

        default_tuple = ()
        for item in text_info_list:
            full_text = item["text"]
            if text in full_text:
                x1, y1, x2, y2 = item["box"]
        
                total_width = x2 - x1
                total_height = y2 - y1
                
                if len(full_text) == 0:
                    continue
                char_width = total_width / len(full_text)
                
                pos = full_text.find(text)
                if pos == -1:
                    continue  
                
                sub_text_width = len(text) * char_width
                offset_x = pos * char_width + sub_text_width / 2
                offset_y = total_height / 2
                
                abs_x = region[0] + x1 + offset_x
                abs_y = region[1] + y1 + offset_y
                
                scaled_x = abs_x / scale
                scaled_y = abs_y / scale
    
                return (float(scaled_x), float(scaled_y))
        return default_tuple


import re
import time,json


class ElementOperation:
    """
        元素操作
        Element operation
    """

    def get_element_name(self, hwnd: str, xpath: str, wait_time: float = 5, interval_time: float = 0.5) -> str:
        """
            获取元素名称
            Get the element name

            hwnd: 窗口句柄
            xpath: 元素路径
            wait_time: 等待时间，默认取 5秒
            interval_time: 轮询间隔时间，默认 0.5秒
            return: 元素名称字符串或 None
        
            hwnd: window handle
            xpath: element path
            wait_time: waiting time, which is 5 seconds by default
            interval_time: the polling interval, which is 0.5 seconds by default
            return: element name string or None
        """

        end_time = time.time() + wait_time
        while time.time() < end_time:
            response = self.SendData("getElementName", hwnd, xpath)
            if response == "null":
                time.sleep(interval_time)
                continue
            else:
                return response
        return None

    def get_element_value(self, hwnd: str, xpath: str, wait_time: float = 5, interval_time: float = 0.5) -> str:
        """
            获取元素文本(可编辑的那种文本)
            Gets the element text (editable text)

            hwnd: 窗口句柄
            xpath: 元素路径
            wait_time: 等待时间，默认取 5秒
            interval_time: 轮询间隔时间，默认取 0.5秒
            return: 元素文本字符串或 None

            hwnd: window handle
            xpath: element path
            wait_time: waiting time, which is 5 seconds by default
            interval_time: the polling interval, which is 0.5 seconds by default
            return: element text string or None
        """
        end_time = time.time() + wait_time
        while time.time() < end_time:
            response = self.SendData("getElementValue", hwnd, xpath)
            if response == "null":
                time.sleep(interval_time)
                continue
            else:
                return response
        return None

    def get_element_rect(self, hwnd: str, xpath: str, wait_time: float = 5, interval_time: float = 0.5) -> tuple:
        """
            获取元素矩形，返回左上和右下坐标
            Gets an element rectangle and returns the upper left and lower right coordinates.

            hwnd: 窗口句柄
            xpath: 元素路径
            wait_time: 等待时间，默认取 self.wait_timeout；
            interval_time: 轮询间隔时间，默认取 self.interval_timeout；
            return: 左上和右下坐标

            hwnd: window handle
            xpath: element path
            wait_time: the waiting time, which defaults to self. wait _ timeout
            interval_time: polling interval; self. interval _ timeout is selected by default
            return: upper left and lower right coordinates

        """
        if wait_time is None:
            wait_time = self.wait_timeout

        if interval_time is None:
            interval_time = self.interval_timeout

        end_time = time.time() + wait_time
        while time.time() < end_time:
            response = self.SendData("getElementRect", hwnd, xpath)
            if response == "-1|-1|-1|-1":
                time.sleep(interval_time)
                continue
            else:
                x1, y1, x2, y2 = response.split("|")
                return ((float(x2)+float(x1))/2,(float(y2)+float(y1))/2)
        return ()

    def get_element_window(self, hwnd: str, xpath: str, wait_time: float = 5, interval_time: float = 0.5) -> str:
        """
            获取元素窗口句柄
            Get the element window handle

            hwnd: 窗口句柄
            xpath: 元素路径
            wait_time: 等待时间，默认取 5秒
            interval_time: 轮询间隔时间，默认取 0.5秒
            return: 元素窗口句柄字符串或 None

            hwnd: window handle
            xpath: element path
            Wait_time: waiting time, which is 5 seconds by default
            interval_time: the polling interval, which is 0.5 seconds by default
            return: element window handle string or None
        """

        end_time = time.time() + wait_time
        while time.time() < end_time:
            response = self.SendData("getElementWindow", hwnd, xpath)
            if response == "null":
                time.sleep(interval_time)
                continue
            else:
                return response
        return None

    def click_element(self, hwnd: str, xpath: str, typ: int, wait_time: float = 5, interval_time: float = 0.5) -> bool:
        """
            点击元素
            Click element

            hwnd: 窗口句柄
            xpath: 元素路径
            typ: 操作类型，单击左键:1 单击右键:2 按下左键:3 弹起左键:4 按下右键:5 弹起右键:6 双击左键:7 双击右键:8
            wait_time: 等待时间，默认取 5秒
            interval_time: 轮询间隔时间，默认取 0.5秒
            return:  成功返回True 失败返回 False

            hwnd: window handle
            xpath: element path
            typ: operation type, click left key: 1 click right key: 2 press left key: 3 popup left key: 4 press right key: 5 popup right key: 6 double-click left key: 7 double-click right key: 8
            wait_time: waiting time, which is 5 seconds by default
            interval_time: the polling interval, which is 0.5 seconds by default
            return: Returns True successfully, and returns False if it fails
        """

        end_time = time.time() + wait_time
        while time.time() < end_time:
            response = self.SendData('clickElement', hwnd, xpath, typ)
            if response == "false":
                time.sleep(interval_time)
                continue
            else:
                return True
        return False

    def invoke_element(self, hwnd: str, xpath: str, wait_time: float = 5, interval_time: float = 0.5) -> bool:
        """
            执行元素默认操作(一般是点击操作)
            Perform the default operation of the element (generally click operation)

            hwnd: 窗口句柄
            xpath: 元素路径
            wait_time: 等待时间，默认取 self.wait_timeout；
            interval_time: 轮询间隔时间，默认取 self.interval_timeout；
            return: 成功返回True 失败返回 False

            hwnd: window handle
            xpath: element path
            wait_time: the waiting time, which defaults to self. wait _ timeout
            interval_time: polling interval; self. interval _ timeout is selected by default
            return: Returns True successfully, and returns False if it fails
        """

        end_time = time.time() + wait_time
        while time.time() < end_time:
            response = self.SendData('invokeElement', hwnd, xpath)
            if response == "false":
                time.sleep(interval_time)
                continue
            else:
                return True
        return False

    def set_element_focus(self, hwnd: str, xpath: str, wait_time: float = 5, interval_time: float = 0.5) -> bool:
        """
            设置元素作为焦点
            Set element as focus

            hwnd: 窗口句柄
            xpath: 元素路径
            wait_time: 等待时间，默认取 5秒
            param interval_time: 轮询间隔时间，默认取 0.5秒
            return: 成功返回True 失败返回 False

            hwnd: window handle
            xpath: element path
            wait_time: waiting time, which is 5 seconds by default
            param interval_time: polling interval, which is 0.5 seconds by default
            return: Returns True successfully, and returns False if it fails
        """

        end_time = time.time() + wait_time
        while time.time() < end_time:
            response = self.SendData('setElementFocus', hwnd, xpath)
            if response == "false":
                time.sleep(interval_time)
                continue
            else:
                return True
        return False

    def set_element_value(self, hwnd: str, xpath: str, value: str, wait_time: float = 5, interval_time: float = 0.5) -> bool:
        """
            设置元素文本
            Set element text

            hwnd: 窗口句柄
            xpath: 元素路径
            value: 要设置的内容
            wait_time: 等待时间，默认取 5秒
            interval_time: 轮询间隔时间，默认取 0.5秒
            return: 成功返回True 失败返回 False

            hwnd: window handle
            xpath: element path
            value: what to set.
            wait_time: waiting time, which is 5 seconds by default
            interval_time: the polling interval, which is 0.5 seconds by default
            return: Returns True successfully, and returns False if it fails
        """
        end_time = time.time() + wait_time
        while time.time() < end_time:
            response = self.SendData('setElementValue', hwnd, xpath, value)
            if response == "false":
                time.sleep(interval_time)
                continue
            else:
                return True
        return False

    def scroll_element(self, hwnd: str, xpath: str, horizontal: int= -1, vertical: int = -1, wait_time: float = 5, interval_time: float = 0.5) -> bool:
        """
            滚动元素
            Rolling element

            hwnd: 窗口句柄
            xpath: 元素路径
            horizontal: 水平百分比 -1不滚动
            vertical: 垂直百分比 -1不滚动
            wait_time: 等待时间，默认取 5秒
            interval_time: 轮询间隔时间，默认取 0.5秒
            return: 成功返回True 失败返回 False

            hwnd: window handle
            xpath: element path
            horizontal: horizontal percentage -1 does not scroll
            vertical: vertical percentage -1 does not scroll
            wait_time: waiting time, which is 5 seconds by default
            interval_time: the polling interval, which is 0.5 seconds by default
            return: Returns True successfully, and returns False if it fails
        """

        end_time = time.time() + wait_time
        while time.time() < end_time:
            response = self.SendData('setElementScroll', hwnd, xpath, horizontal, vertical)
            if response == "false":
                time.sleep(interval_time)
                continue
            else:
                return True
        return False

    def is_selected(self, hwnd: str, xpath: str, wait_time: float = 5, interval_time: float = 0.5) -> bool:
        """
            单/复选框是否选中
            Is the single/check box selected

            hwnd: 窗口句柄
            xpath: 元素路径
            wait_time: 等待时间， 默认取 5秒
            interval_time: 轮询间隔时间，默认取 0.5秒
            return:  成功返回True 失败返回 False

            Hwnd: window handle
            Xpath: element path
            Wait_time: waiting time, which is 5 seconds by default
            Interval_time: the polling interval, which is 0.5 seconds by default
            Return: Returns True successfully, and returns False if it fails
        """
        end_time = time.time() + wait_time
        while time.time() < end_time:
            response = self.SendData('isSelected', hwnd, xpath)
            if response == "false":
                time.sleep(interval_time)
                continue
            else:
                return True
        return False

    def close_window(self, hwnd: str, xpath: str) -> bool:
        """
            关闭窗口
            Close the window

            hwnd: 窗口句柄
            xpath: 元素路径
            return:  成功返回True 失败返回 False

            hwnd: window handle
            xpath: element path
            return: Returns True successfully, and returns False if it fails
        """
        return "true" in self.SendData('closeWindow', hwnd, xpath) 

    def set_element_state(self, hwnd: str, xpath: str, state: str) -> bool:
        """
            设置窗口状态
            Set window state

            hwnd: 窗口句柄
            xpath: 元素路径
            state: 0正常 1最大化 2 最小化
            return:  成功返回True 失败返回 False

            hwnd: window handle
            xpath: element path
            state: 0 Normal 1 Maximize 2 Minimize
            return: Returns True successfully, and returns False if it fails
        """
        return "true" in self.SendData('setWindowState', hwnd, xpath, state)
    
    def get_elements(self, hwnd) -> json:
        """
            获取可见区域内的所有元素信息
            Get information of all elements in the visible area

            hwnd: 窗口句柄
            return: 成功返回数组json格式的元素信息，失败返回null

            hwnd: window handle
            return: element information in json format of array is returned successfully, and null is returned if it fails
        """
        response = self.SendData("getElements", hwnd)
        if response == "null":
            return None
        return json.loads(response)


import re


class PagesNavigation:
    """
        页面和导航功能模块
        Page and navigation function module
    """

    def goto(self, url: str) -> bool:
        """
            跳转页面
            Jump page

            url: 网址
            return: 布尔值

            url: website address
            return: bool
        """
        return "true" in self.SendData("goto", url)

    def new_page(self, url: str) -> bool:
        """
            新建 Tab 并跳转页面
            Create a new Tab and jump to the page

            url: 网址
            return: 布尔值

            url: website address
            return: bool
        """
        return "true" in self.SendData("newPage", url)

    def back(self) -> bool:
        """
            后退
            go back

            return: 布尔值
            return: bool
        """
        return "true" in self.SendData("back")

    def forward(self) -> bool:
        """
            前进
            go move

            return: 布尔值
            return: bool
        """
        return "true" in self.SendData("forward")

    def refresh(self) -> bool:
        """
            刷新
            refresh

            return: 布尔值
            return: bool
        """
        return "true" in self.SendData("refresh")

    def get_current_page_id(self) -> str:
        """
            获取当前页面 ID
            Get the current page ID

            return: 字符串, 找不到页面则返回None
            return: str or None if not found Page
        """
        response = self.SendData("getCurPageId")
        if response == "null":
            return None
        return response

    def get_all_page_id(self) -> list:
        """
            获取所有页面 ID
            Get all page ids

            return: 所有页面ID，找不到则返回空列表[]
            return: all page ID, or an empty list [] if it is not found
        """
        response = self.SendData("getAllPageId")
        if response == "null":
            return []
        return response.split("|")
    
    def switch_to_page(self, page_id: str) -> bool:
        """
            切换到指定页面
            Switch to the specified page

            page_id: 你要切换的页面ID
            return: 布尔值

            page_id: ID of the page you want to switch.
            return: bool
        """
        return "true" in self.SendData("switchPage", page_id)

    def close_current_page(self) -> bool:
        """
            关闭当前页面
            Close the current page

            return: 布尔值
            return: bool
        """
        return "true" in self.SendData("closePage")

    def get_current_url(self) -> str:
        """
            获取当前页面 URL
            Get URL of current page

            return: 当前页面URL 或 None
            return: URL of current page or None
        """
        response = self.SendData("getCurrentUrl")
        if response == "webdriver error":
            return None
        response = re.findall(r'(http.*)',response)
        if response:
            return response[0]
        else:
            return None

    def get_current_title(self) -> str:
        """
            获取当前页面标题
            Get the current page title

            return：当前页面标题 或 None
            return：current page title or None

        """
        response = self.SendData("getTitle")
        if response == "webdriver error":
            return None
        return response

    def enable_network(self, url: str, method: str) -> bool:
        """
            启用网络数据监听，此函数一般在goto、点击跳转等操作 前调用。可多次调用监听多个目标
            Enable network data monitoring. This function is generally called before operations such as goto and click jump
            Multiple calls can be made to monitor multiple targets

            url: 监听的url
            method: 监听的请求类型。'GET'或'POST'
            return: 总是返回 true

            url: url for listening
            method: the type of request to listen to. GET' or' POST'
            return: always returns true

        """
        return "true" in self.SendData("enableNetwork", url, method)

    def wait_network_data(self, timeout : int = 10) -> str:
        """
            等待获取网络数据，此函数必须在 goto、点击跳转等操作后立刻执行。执行完自动关闭 enableNetwork 设置的监听目标
            Waiting to obtain network data, this function must be executed immediately after goto, click jump and other operations
            After performing automatic shutdown of the listening target set by enableNetwork

            timeout: 等待超时,单位毫秒。默认10秒
            return: 成功返回json数组格式的数据，失败返回null

            timeout: Wait timeout, in milliseconds. Default 10 seconds
            return: data in jsons array format is returned successfully, and null is returned if it fails

        """
        return self.SendData("waitNetworkData", timeout*1000)




        


"""
    PyAibote Web自动化框架，由WebDriver.exe
    客户端驱动程序连接脚本服务端，采用TCP协议传输命令。
    底层基于chromium内核研发而成的一款web自动化框架,
    因此支持chromium内核的所有浏览器和应用程序, 框架稳定, 运行速度非常快.

    PyAibote Web automation framework, 
    with WebDriver.exe client driver connected to script server and TCP protocol used to transmit commands.
    The bottom layer is a web automation framework based on the chromium kernel, so it supports all browsers and applications of the chromium kernel, 
    and the framework is stable and runs very fast.
"""


import os,sys
import time,subprocess,json,random
from multiprocessing import Process
from abc import ABC, abstractmethod
import socketserver,socket
from .WebBotModel import  *
from .Tool import  *


class WebBotMain(
        ABC,
        LoggerRecord,
        ThreadingTCPServer, 
        UniversalFunction,
        SendClientData, 
        WebLoadWait,
        PagesNavigation,
        ElementOperation,
        KeymouseOperation,
        PopUpWindow,
        WindowOperation,
        DrivingOperation,
        JSinjection,
        CookiesOperation,
        ChatGenerative,
        DataBaseHandle,
        Sqlite3DataBaseHandle,
        IframeOperation,
        WebSocketServerUse
    ):

    def __init__(self,*args):
        if len(args) ==1:
            address_info = socket.getaddrinfo(None, args[0], socket.AF_INET, socket.SOCK_STREAM)[0]
            family, socket_type, proto, _, socket_address = address_info
            server = socket.socket(family, socket_type, proto)
            server.bind(socket_address)
            server.listen(1)
            print("Mix Start WebBot Service")
            self.request, self.client_address = server.accept()
            print("WebBot Client link succeeded")
        else:
            super().__init__(*args)

    @classmethod
    def _build(self, listen_ip, listen_port: int, Debug: bool = True, driver_params: dict = {}) -> object:
        """
            混合开发时启动web驱动
            TCP transit service.

            listen_port: 脚本监听的端口
            return: windows类对象

            listen_port: the port on which the script listens.
            Debug: Is the script deployed locally?
            return: windows class object
        """
        if Debug:
            default_params = {
                "serverIp": "127.0.0.1",
                "serverPort": listen_port,
                "browserName": "chrome",
                "debugPort": 0,
                "userDataDir": f"./UserData{random.randint(100000, 999999)}",
                "browserPath": None,
                "argument": None,
            }
            if driver_params:
                default_params.update(driver_params)
            default_params = json.dumps(default_params)
            
            try:
                print("WebDriver And Windows Mix Model Start WebDriver ...")
                subprocess.Popen(["WebDriver.exe", default_params])
                print("Mix Start WebDriver Successful，Execute Script")
            except FileNotFoundError as e:
                err_msg = "\nStart local WebDriver.exe fail Exception elimination step：\n1. Check WebDriver.exe Path；\n2. WebDriver.exe Add to system environment variable?"
                self.error(f"{err_msg}: " + str(e))

        return WebBotMain(listen_port)

    # @abstractmethod
    def script_main(self):
        pass
    
    def handle(self):
        self.info(f"<-<- Client connection at {self.client_address[0]}: {self.client_address[1]}")
        self.debug(f"<-<- Client connection at {self.client_address[0]}: {self.client_address[1]}")
        self.script_main()

    @classmethod
    def execute(self, IP: str, Port: int, Debug: bool = True, Driver_Params: dict = None, Qt = None, WebsocketSwitch = False, WebsocketPort = 8888):
        try:
            if Port < 0 or Port > 65535:
                raise OSError("`listen_port` must be in 0-65535.")

            if Qt:
                self.Qt = Qt

            if Debug:
                Driver.WebDriverStart(Port, Driver_Params)

            if WebsocketSwitch:
                server = WebSocketServerThread(IP, WebsocketPort, self.Websocket_Log_Level)
                server.start()

            ThreadingTCPServer.StartThreadingTCPServer(self, IP, Port)
        except KeyboardInterrupt as e:
            sys.exit(self)


    @classmethod
    def StopSrver(self):
        try:
            self.server.shutdown()  # 停止接受新的连接
            self.server.socket.close()  # 关闭服务器套接字
        except Exception as e:
            pass
    


import traceback

class SendClientData:
    """
        将TCP数据发送给驱动程序
        Give TCP data to the driver
    """

    def SendData(self, *args) -> str:
        args_len = ""
        args_text = ""
        
        for argv in args:
            if argv is None:
                argv = ""
            elif isinstance(argv, bool) and argv:
                argv = "true"
            elif isinstance(argv, bool) and not argv:
                argv = "false"
            argv = str(argv)
            args_text += argv
            args_len += str(len(bytes(argv, 'utf8'))) + "/"
        data = (args_len.strip("/") + "\n" + args_text).encode("utf8")
        try:
            response = self.StarLoadWait(data)
            return response
        except Exception as e:
            self.request.close()
            self.error("send/read tcp data error: " + str(traceback.format_exc()))
            raise e




class OtherOperations:
    """
        其他操作
        OtherOperations
    """

    def download_file(self, url: str, file_path: str, is_wait: bool) -> bool:
        """
            下载文件
            Download file

            url: 文件地址
            file_path: 文件保存的路径
            is_wait: 是否等待下载完成
            return:  总是返回True

            url: file address
            file_path: the path where the file is saved
            is_wait: Do you want to wait for the download to complete
            return: always returns True
        """
        return "true" in self.SendData("downloadFile", url, file_path, is_wait)




import re
import json

class CookiesOperation:
    """
         Cookies操作
         CookiesOperation
    """

    def get_cookies(self, url: str) -> list:
        """
            获取指定 url 的 Cookies
            Get Cookies for the specified url

            url: url 字符串
            return: 布尔值

            url: url string
            return: bool
        """
        response = self.SendData("getCookies", url)
        response = response.replace("\n","").replace("\t","")
        if response == "null":
            return None
        return json.loads(response)

    def get_all_cookies(self) -> list:
        """
             获取浏览器所有的Cookies
            Get all the Cookies

            return: 列表格式的 cookies
            return: cookies in list format
        """
        response = self.SendData("getAllCookies")
        response = response.replace("\n","").replace("\t","")
        if response == "null":
            return None
        return json.loads(response)

    def set_cookies(self, url: str, name: str, value: str, options: dict = None) -> bool:
        """
            设置指定 url 的 Cookies
            Set Cookies for the specified url

            url: 要设置 Cookie 的域
            name: Cookie 名
            value: Cookie 值
            options: 其他属性
            return: 布尔值

            url: The domain to set the Cookie
            name: Cookie name
            value: Cookie value
            options: Other attributes
            return: bool
        """
        default_options = {
            "domain": "",
            "path": "",
            "secure": False,
            "httpOnly": False,
            "sameSite": "",
            "expires": 0,
            "priority": "",
            "sameParty": False,
            "sourceScheme": "",
            "sourcePort": 0,
            "partitionKey": "",
        }
        if options:
            default_options.update(options)
        return "true" in self.SendData("setCookie", name, value, url, *default_options.values())

    def delete_cookies(self, name: str, url: str = "", domain: str = "", path: str = "") -> bool:
        """
            删除指定 Cookie
            Delete the specified Cookie

            name: 要删除的 Cookie 的名称
            url: 删除所有匹配 url 和 name 的 Cookie
            domain: 删除所有匹配 domain 和 name 的 Cookie
            path: 删除所有匹配 path 和 name 的 Cookie
            return: 布尔值

            name: The name of the Cookie to delete
            url: Delete all Cookie that match url and name
            domain: Delete all Cookie matching domain and name
            path: Delete all Cookie matching path and name
            return: bool

        """
        return "true" in self.SendData("deleteCookies", name, url, domain, path)

    def delete_all_cookies(self) -> bool:
        """
            删除所有 Cookie
            Delete all Cookie

            return: 布尔值
            return: bool
        """
        return "true" in self.SendData("deleteAllCookies")

    def clear_cache(self) -> bool:
        """
            清除缓存
            Clear cache

            return: 布尔值
            return: bool
        """
        return "true" in self.SendData("clearCache") 








import re


class JSinjection:
    """
        JS 注入
        JS injection
    """

    def execute_script(self, script: str) -> str:
        """
            注入执行 JS
            Injection execution JS

            script: 要执行的 JS 代码
            return: 假如注入代码有返回值，则返回此值，否则返回 None

            script: JS code to be executed
            return: If the injection code has a return value, this value is returned, otherwise None is returned
            
            示例：
            Examples:
                result = execute_script('(()=>alert(123))()')
        """
        response = self.SendData("executeScript", script)
        if response == "null":
            return None
        return response







import re
import time

class ColorOperation:
    """
        图色操作
        Color operation
    """
    def save_screenshot(self, hwnd: str, save_path: str, region: tuple = (0,0,0,0), algorithm: tuple = (0,0,0), mode: bool = False) -> bool:
        """
            截图
            screenshot

            hwnd: 窗口句柄
            save_path: 图片存储路径
            region: 截图区域，默认全屏，``region = (起点x、起点y、终点x、终点y)``，得到一个矩形
            algorithm:
                处理截图所用算法和参数，默认保存原图，
                ``algorithm = (algorithm_type, threshold, max_val)``
                按元素顺序分别代表：
                0. ``algorithm_type`` 算法类型
                1. ``threshold`` 阈值
                2. ``max_val`` 最大值

                ``threshold`` 和 ``max_val`` 同为 255 时灰度处理.
                    ``algorithm_type`` 算法类型说明:
                        0. ``THRESH_BINARY``      算法，当前点值大于阈值 `threshold` 时，取最大值 ``max_val``，否则设置为 0；
                        1. ``THRESH_BINARY_INV``  算法，当前点值大于阈值 `threshold` 时，设置为 0，否则设置为最大值 max_val；
                        2. ``THRESH_TOZERO``      算法，当前点值大于阈值 `threshold` 时，不改变，否则设置为 0；
                        3. ``THRESH_TOZERO_INV``  算法，当前点值大于阈值 ``threshold`` 时，设置为 0，否则不改变；
                        4. ``THRESH_TRUNC``       算法，当前点值大于阈值 ``threshold`` 时，设置为阈值 ``threshold``，否则不改变；
                        5. ``ADAPTIVE_THRESH_MEAN_C``      算法，自适应阈值；
                        6. ``ADAPTIVE_THRESH_GAUSSIAN_C``  算法，自适应阈值；

            mode: 操作模式，后台 true，前台 false, 默认前台操作
            return: "False"或者 "True"

            hwnd: window handle
            save_path: the image storage path.
            region: screenshot area, full screen by default, `` region = (starting point x, starting point y, ending point x, ending point y) ```, and a rectangle is obtained.
            algorithm:
                The algorithm and parameters used to process screenshots, the original image is saved by default,
                ``algorithm = (algorithm_type, threshold, max_val)``
                Represents, respectively, in the order of elements:
                0. ``algorithm_type ``` algorithm type
                1. ``threshold ```` threshold
                2. `` Max _ val```` Maximum value

                Gray processing when threshold and max_val are both 255.
                    `` algorithm _ type``` description of algorithm_type:
                        0. ``` threshold _ binary``` algorithm, when the current point value is greater than the threshold value ` threshold `, the maximum value ``` max _ val```` is taken, otherwise it is set to 0;
                        1. `` threshold _ binary _ inv``` algorithm, when the current point value is greater than the threshold value ` threshold `, it is set to 0, otherwise it is set to the maximum value max_val；;
                        2. `` threshold _ tozero``` algorithm, when the current point value is greater than the threshold ` threshold', it will not be changed, otherwise it will be set to 0;
                        3. `` threshold _ tozero _ inv``` algorithm, when the current point value is greater than the threshold ``` threshold ```, set it to 0, otherwise it will not change;
                        4. `` threshold _ trunc``` algorithm, when the current point value is greater than the threshold value ``` threshold ```, it is set as the threshold value ``` threshold ```, otherwise it will not be changed;
                        5. ``` Adaptive _ threshold _ mean _ c``` algorithm, adaptive threshold;
                        6. ``` Adaptive _ Thresh _ Gaussian _ C``` algorithm, adaptive threshold;

            mode: operation mode, background true, foreground false, default foreground operation.
            return: "False "or" True "
        """

        algorithm_type, threshold, max_val = algorithm
        if algorithm_type in (5, 6):
            threshold = 127
            max_val = 255

        return "true" in self.SendData("saveScreenshot", hwnd, save_path, *region, algorithm_type, threshold, max_val,mode)

    def get_color(self, hwnd: str, x: float, y: float, mode: bool = False) -> str:
        """
            获取指定坐标点的色值
            Gets the color value of the specified coordinate point and returns the color value string (#008577) or None

            hwnd: 窗口句柄
            x: x 坐标；
            y: y 坐标；
            mode: 操作模式，后台 True，前台 False, 默认前台操作
            return: 色值字符串(#008577)或者 None

            hwnd: Window handle
            x: x coordinate
            y: y coordinate
            mode: Operation mode, background True，前台, foreground False, default foreground operation.
            return: Color value string (#008577) or None
        """
        response = self.SendData("getColor", hwnd, x, y, mode)
        if response == "null":
            return None
        return response

    def find_color(self, hwnd: str, color: str, sub_colors: tuple = (), region: tuple = (0, 0, 0, 0),similarity: float = 0.9, mode: bool = False, wait_time: float = 5, interval_time: float = 0.5)  -> tuple:
        """
            获取指定色值的坐标点，返回坐标或者 None
            Gets the coordinate point of the specified color value, and returns the coordinate or None

            hwnd: 窗口句柄
            color: 颜色字符串，必须以 # 开头，例如：#008577
            sub_colors: 辅助定位的其他颜色
            region: 在指定区域内找色，默认全屏
            similarity: 相似度，0-1 的浮点数，默认 0.9
            mode: 操作模式，后台 true，前台 false, 默认前台操作
            wait_time: 等待时间，默认取 self.wait_timeout
            interval_time: 轮询间隔时间，默认取 self.interval_timeout
            return: 成功返回{x:number, y:number} 失败返回None


            hwnd: window handle
            color: color string, which must start with #, for example: #008577
            sub_colors: other colors to assist positioning
            region: Find the color in the specified area, full screen by default
            similarity: similarity, floating point number of 0-1, default is 0.9
            mode: operation mode, background true, foreground false, default foreground operation
            wait_time: the waiting time, which is self.wait_timeout by default
            interval_time: polling interval; self.interval_timeout is selected by default
            return: returned {x:number, y:number} successfully, and returned None if failed
        """

        if sub_colors:
            sub_colors_str = ""
            for sub_color in sub_colors:
                offset_x, offset_y, color_str = sub_color
                sub_colors_str += f"{offset_x}/{offset_y}/{color_str}\n"
            sub_colors_str = sub_colors_str.strip()
        else:
            sub_colors_str = "null"

        end_time = time.time() + wait_time
        while time.time() < end_time:
            response = self.SendData("findColor", hwnd, color, sub_colors_str, *region, similarity, mode)
            if response == "-1.0|-1.0":
                time.sleep(interval_time)
            else:
                x, y = response.split("|")
                return (x,y)

        return None

    def compare_color(self, hwnd: str, main_x: float, main_y: float, color: str, sub_colors: tuple = (), region: tuple = (0, 0, 0, 0), similarity: float = 0.9, mode: bool = False) -> bool:
        """
            比较指定坐标点的颜色值
            Compare the color values of the specified coordinate points

            hwnd: 窗口句柄；
            main_x: 主颜色所在的X坐标；
            main_y: 主颜色所在的Y坐标；
            color: 颜色字符串，必须以 # 开头，例如：#008577；
            sub_colors: 辅助定位的其他颜色；
            region: 截图区域，默认全屏，``region = (起点x、起点y、终点x、终点y)``，得到一个矩形
            similarity: 相似度，0-1 的浮点数，默认 0.9；
            mode: 操作模式，后台 true，前台 false, 默认前台操作；
            return: True或者 False

            hwnd: window handle
            main_x: the x coordinate of the main color
            main_y: the y coordinate of the main color
            color: color string, which must start with #, for example: # 008577
            sub_colors: other colors to assist positioning
            region: screenshot area, full screen by default, `` region = (starting point x, starting point y, ending point x, ending point y) ```, and a rectangle is obtained
            similarity: similarity, floating point number of 0-1, default 0.9
            mode: operation mode, background true, foreground false, default foreground operation
            return: True or False
        """

        if sub_colors:
            sub_colors_str = ""
            for sub_color in sub_colors:
                offset_x, offset_y, color_str = sub_color
                sub_colors_str += f"{offset_x}/{offset_y}/{color_str}\n"
            sub_colors_str = sub_colors_str.strip()
        else:
            sub_colors_str = "null"
        return "true" in self.SendData("compareColor", hwnd, main_x, main_y, color, sub_colors_str, *region, similarity,mode)

    def extract_image_by_video(self, video_path: str, save_folder: str, jump_frame: int = 1) -> bool:
        """
            提取视频帧
            Extract video frames

            video_path: 视频路径
            save_folder: 提取的图片保存的文件夹目录
            jump_frame: 跳帧，默认为1 不跳帧
            return: True或者False

            video_path: Video path
            save_folder: The folder directory where the extracted pictures are saved
            jump_frame: Frame skipping, the default is 1, no frame skipping
            return: True or False
        """
        return "true" in self.SendData("extractImageByVideo", video_path, save_folder, jump_frame)

    def crop_image(self, image_path, save_path, left, top, rigth, bottom) -> bool:
        """
            裁剪图片
            Crop a picture

            image_path: 图片路径
            save_path: 裁剪后保存的图片路径
            left: 裁剪的左上角横坐标
            top: 裁剪的左上角纵坐标
            rigth: 裁剪的右下角横坐标
            bottom: 裁剪的右下角纵坐标
            return: True或者False

            image_path: image path
            save_path: the path of the saved picture after cropping
            left: abscissa of the upper left corner of the crop
            top: the ordinate of the upper left corner of the crop
            rigth: abscissa of the lower right corner of the crop
            bottom: ordinate of the lower right corner of the crop
            return: True or False
        """
        return "true" in self.SendData("cropImage", image_path, save_path, left, top, rigth, bottom) 

    def find_images(self, hwnd_or_big_image_path: str, image_path: str, region: tuple = (0, 0, 0, 0), algorithm: tuple = (0, 0, 0), similarity: float = 0.9, mode: bool = False, multi: int = 1, wait_time: float = 5, interval_time: float = 0.5) -> list:
        """
            寻找图片坐标，在当前屏幕中寻找给定图片中心点的坐标，返回坐标列表
            Find picture coordinates, find the coordinates of the center point of a given picture in the current screen, and return to the coordinate list

            hwnd_or_big_image_path: 窗口句柄或者图片路径；
            image_path: 图片的绝对路径；
            region: 从指定区域中找图，默认全屏；
            algorithm: 处理屏幕截图所用的算法，默认原图，注意：给定图片处理时所用的算法，应该和此方法的算法一致；
            similarity: 相似度，0-1 的浮点数，默认 0.9；
            mode: 操作模式，后台 true，前台 false, 默认前台操作；
            multi: 返回图片数量，默认1张；
            wait_time: 等待时间，默认取 self.wait_timeout；
            interval_time: 轮询间隔时间，默认取 self.interval_timeout；
            return: 成功返回 单坐标点[{x:number, y:number}]，多坐标点[{x1:number, y1:number}, {x2:number, y2:number}...] 失败返回空[]

            thresholdType算法类型：
                0   THRESH_BINARY算法，当前点值大于阈值thresh时，取最大值maxva，否则设置为0
                1   THRESH_BINARY_INV算法，当前点值大于阈值thresh时，设置为0，否则设置为最大值maxva
                2   THRESH_TOZERO算法，当前点值大于阈值thresh时，不改变，否则设置为0
                3   THRESH_TOZERO_INV算法，当前点值大于阈值thresh时，设置为0，否则不改变
                4   THRESH_TRUNC算法，当前点值大于阈值thresh时，设置为阈值thresh，否则不改变
                5   ADAPTIVE_THRESH_MEAN_C算法，自适应阈值
                6   ADAPTIVE_THRESH_GAUSSIAN_C算法，自适应阈值
                thresh阈值，maxval最大值，threshold默认保存原图。thresh和maxval同为255时灰度处理

            hwnd_or_big_image_path: window handle or image path;
            image_path: the absolute path of the picture;
            region: Find the map from the specified area, full screen by default;
            algorithm: the algorithm used to process screen shots, the original image by default. Note: the algorithm used in a given image processing should be consistent with the algorithm of this method;
            similarity: similarity, floating point number of 0-1, default 0.9;
            mode: operation mode, background true, foreground false, default foreground operation;
            multi: Returns the number of pictures, with 1 picture by default;
            wait_time: the waiting time, which defaults to self. wait _ timeout;
            interval_time: polling interval; self. interval _ timeout is selected by default;
            return: single coordinate point [{x:number, y:number}] was successfully returned, and multi-coordinate point [{x1: number, y1: number}, {x2: number, y2: number} ...] failed to return empty [].

            ThresholdType algorithm type:
                0 THRESH_BINARY algorithm, when the current point value is greater than the threshold thresh, take the maximum maxva, otherwise set to 0
                1 THRESH_BINARY_INV algorithm, when the current point value is greater than the threshold thresh, set it to 0, otherwise set it to the maximum maxva
                2 THRESH_TOZERO algorithm, when the current point value is greater than the threshold thresh, it will not change, otherwise it will be set to 0
                3 THRESH_TOZERO_INV algorithm, when the current point value is greater than the threshold thresh, it is set to 0, otherwise it will not change
                4 THRESH_TRUNC algorithm, when the current point value is greater than the threshold thresh, set it as the threshold thresh, otherwise it will not change
                5 ADAPTIVE_THRESH_MEAN_C algorithm, adaptive threshold
                6 adaptive _ threshold _ Gaussian _ c algorithm, adaptive threshold
                Threshold threshold, maxval maximum, threshold saves the original image by default. Gray processing when thresh and maxval are both 255
        """

        algorithm_type, threshold, max_val = algorithm
        if algorithm_type in (5, 6):
            threshold = 127
            max_val = 255

        end_time = time.time() + wait_time
        while time.time() < end_time:
            if hwnd_or_big_image_path.isdigit():
                response = self.SendData("findImage", hwnd_or_big_image_path, image_path, *region, similarity,
                                            algorithm_type,
                                            threshold, max_val, multi, mode)
            else:
                response = self.SendData("findImageByFile", hwnd_or_big_image_path, image_path, *region, similarity,
                                            algorithm_type,
                                            threshold, max_val, multi, mode)
            if response in ["-1.0|-1.0", "-1|-1"]:
                time.sleep(interval_time)
                continue
            else:
                image_points = response.split("/")
                point_list = []
                for point_str in image_points:
                    x, y = point_str.split("|")
                    point_list.append((float(x), float(y)))
                return point_list
        return []

    def find_dynamic_image(self, hwnd: str, interval_ti: int, region: tuple = (0, 0, 0, 0), mode: bool = False, wait_time: float = 5, interval_time: float = 0.5) -> list:
        """
            找动态图，对比同一张图在不同时刻是否发生变化，返回坐标列表
            Find the dynamic diagram, compare whether the same diagram has changed at different times, and return the coordinate list

            hwnd: 窗口句柄
            interval_ti: 前后时刻的间隔时间，单位毫秒
            region: 在指定区域找图，默认全屏
            mode: 操作模式，后台 true，前台 false, 默认前台操作
            wait_time: 等待时间，默认取 self.wait_timeout
            interval_time: 轮询间隔时间，默认取 self.interval_timeout
            return: 成功返回 单坐标点[{x:number, y:number}]，多坐标点[{x1:number, y1:number}, {x2:number, y2:number}...] 失败返回空[]

            hwnd: window handle
            interval_ti: the interval between the preceding and the following moments, in milliseconds
            region: Find the map in the specified area, full screen by default
            mode: operation mode, background true, foreground false, default foreground operation
            wait_time: the waiting time, which is self.wait_timeout by default
            interval_time: polling interval; self.interval_timeout is selected by default
            return: single coordinate point [{x:number, y:number}] was successfully returned, and multi-coordinate point [{x1: number, y1: number}, {x2: number, y2: number} ...] failed to return empty []
        """

        end_time = time.time() + wait_time
        while time.time() < end_time:
            response = self.SendData("findAnimation", hwnd, interval_ti, *region, mode)
            if response == "-1|-1":
                time.sleep(interval_time)
                continue
            else:
                image_points = response.split("/")
                point_list = []
                for point_str in image_points:
                    x, y = point_str.split("|")
                    point_list.append((float(x), float(y)))
                return point_list
        return []







"""
	写自定义LOG文件
    Write a custom LOG file
"""

import logging
import os
from loguru import logger as lg

class LoggerRecord(object):
    def Custom_Write_logger(self, LogName, FileDirName, LogFileName, Log_Print, LogType, Info)  -> bool:
        """
            自定义写Log文件
            Custom write Log file

            LogName: 类型名称区分那种的log
            FileDirName: log文件夹路径
            LogFileName: log文件名称
            Info: log的信息
            Log_Print: 是否通过终端打印log信息
            LogType: log的类型 error, info
            return: 成功返回True or 失败返回False

            LogName: the type name distinguishes which kind of log
            FileDirName: log folder path
            LogFileName: log file name
            Info: log information
            Log_Print: whether the Log_Print: Log log information through the terminal
            return: Returns True or success or False on failure
        """
        try:
            if not os.path.exists(FileDirName):
                os.makedirs(FileDirName)
            logger = logging.getLogger(LogName)
            logger.setLevel(logging.DEBUG)# 等级为DEBUG
            fh = logging.FileHandler(f'{FileDirName}{LogFileName}',encoding="utf-8",mode="a")
            fh.setLevel(logging.DEBUG)
            fh_formatter = logging.Formatter('%(asctime)s - %(filename)s[line:%(lineno)d] - %(levelname)s : %(message)s')
            if Log_Print:
                if LogType == "error":
                    lg.error(Info)
                elif LogType == "info":
                    lg.info(Info)

            fh.setFormatter(fh_formatter)
            logger.addHandler(fh)
            logger.debug(Info)
            logger.removeHandler(fh)
            return True
        except Exception as e:
            return False
        


import re
import json

class WindowOperation:
    """
        窗口操作
        Window operation
    """

    def get_window_pos(self) -> dict:
        """
            获取窗口位置和状态
            Get the window position and status.

            return: left和top：浏览器相对于与Windows窗口左上角坐标点，height和width：浏览器本身高度和宽度
            return: left and top: the coordinates of the browser relative to the upper left corner of the Windows window, height and width: the height and width of the browser itself.
        """

        response = self.SendData("getWindowPos")
        response = response.replace("\n","").replace("\t","")

        if response == "null":
            return None
        response = json.loads(response)
        return response

    def set_window_pos(self, status: str = "normal", left: float = 0, top: float = 0, width: float = 0, height: float = 0) -> bool:
        """
            设置窗口位置和窗口大小窗口状态
            Set window position and window size window state

            status: 正常:"normal"  最小化:"minimized"  最大化:"maximized"  全屏:"fullscreen"
            left:   浏览器相对于与Windows窗口左上角X坐标点     可选参数，浏览器窗口位置，此参数仅 status 值为 "normal" 时有效
            top:    浏览器相对于与Windows窗口左上角Y坐标点     可选参数，浏览器窗口位置，此参数仅 status 值为 "normal" 时有效
            width:  浏览器本身宽度                           可选参数，浏览器窗口位置，此参数仅 status 值为 "normal" 时有效
            height: 浏览器本身高度                           可选参数，浏览器窗口位置，此参数仅 windowState 值为 "normal" 时有效
            
            return: 布尔值

            left: an optional parameter, browser window position, relative to the X coordinate point in the upper left corner of the Windows window. This parameter is only valid when the status value is "normal"
            top: the optional parameter of the browser relative to the Y coordinate point in the upper left corner of the Windows window, and the browser window position. This parameter is only valid when the status value is "normal"
            width: optional parameter of browser width and browser window position. This parameter is only valid when the status value is "normal"
            height: optional parameter of browser height and browser window position. This parameter is only valid when the value of windowState is "normal"
            status: normal: "normal" minimize: "minimized" maximize: "maximized" fullscreen: "full screen"
            return: bool
        """
        return "true" in self.SendData("setWindowPos", status, left, top, width, height) 

    def quit(self) -> bool:
        """
            退出浏览器
            Exit the browser

            return: 布尔值
            return: bool
        """
        return "true" in self.SendData("closeBrowser")
        
    def mobile_emulation(self, width: int, height: int, userAgent: str, platform: str, platformVersion: str, acceptLanguage: str = "", timezoneId: str = "", latitude: float = 0, longitude: float = 0,accuracy: float = 0) -> bool:
        """
            模拟移动端浏览器
            Analog mobile browser

            width: 宽度
            height: 高度
            userAgent: 用户代理
            platform: 系统平台，例如 "Android"、"IOS"、"iPhone"
            platformVersion: 系统版本号，例如 "9.0"，应当与userAgent提供的版本号对应
            acceptLanguage: 可选参数，语言，例如 "zh-CN"、"en"
            timezoneId: 可选参数，时区标识，例如"Asia/Shanghai"、"Europe/Berlin"、"Europe/London" 时区应当与 语言、经纬度 对应
            latitude: 可选参数，纬度，例如 31.230416
            longitude: 可选参数，经度，例如 121.473701
            accuracy: 可选参数，精度，例如 1111
            return: 布尔值

            width: width
            height: height
            userAgent: user agent
            platform: System platform, such as "Android", "IOS" and "iPhone"
            platformVersion: the system version number, such as "9.0", should correspond to the version number provided by userAgent
            acceptLanguage: optional parameter and language, such as "zh-CN" and "en"
            timezoneId: optional parameter, time zone identifier, such as "Asia/Shanghai", "Europe/Berlin" and "Europe/London", the time zone should correspond to the language, latitude and longitude
            latitude: optional parameter, latitude, for example, 31.416
            longitude: optional parameter, longitude, for example, 121.01
            accuracy: optional parameter, precision, such as 1111
            return: Boolean value.

            示例：
            Example:
                self.mobile_emulation(500, 800, "Mozilla/5.0 (Linux; Android 7.0; SM-G950U Build/NRD90M) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/62.0.3202.84 Mobile Safari/537.36", 
                    "Android", "7.0", "zh-Hans-CN", "Asia/Shanghai", 31.230416, 121.473701, 1111)
        """
        return "true" in self.SendData("mobileEmulation", width, height, userAgent, platform, platformVersion, acceptLanguage, timezoneId, latitude, longitude,accuracy) 



        


import re




class AndroidHidCorrelation:
    """
        hid相关
        Hid correlation
    """

    def __init_accessory(self) -> bool:
        """
            初始化 android Accessory，获取手机 hid 相关的数据
            Initialize android Accessory to get the data related to mobile phone hid

            return: True或者False
            return: True or False.
        """
        return "true" in self.SendData("initAccessory") 

    def init_hid(self, win_driver) -> bool:
        """
            初始化Hid,不能重复调用，重复调用会导致get_hid_data取不到数据
            Initialize Hid, and you can't call it repeatedly. Repeated calls will cause get_hid_data to get no data
            
            hid实际上是由windowsBot 通过数据线直接发送命令给安卓系统并执行，并不是由aibote.apk执行的命令。
            我们应当将所有设备准备就绪再调用此函数初始化。
            Windows initHid 和 android initAccessory函数 初始化目的是两者的数据交换，并告知windowsBot发送命令给哪台安卓设备
            win_driver: windowsDriver 实例，是调用 build_win_driver 的返回值
            return: True或者False

            In fact, hid is a command directly sent by windowsBot to Android system through data line and executed, not by aibote.apk
            We should get all the devices ready before calling this function to initialize
            The initialization purpose of Windows initHid and android initAccessory functions is to exchange data between them and tell windowsBot which Android device to send commands to
            Win_driver: windowsDriver instance, which is the return value of calling build_win_driver
            return: True or False
        """
        
        self.android_id = self.get_android_id()
        self.win_driver = win_driver
        if not self.win_driver:
            return False
        if not self.__init_accessory():
            return False
        self.android_ids = self.win_driver.get_hid_data()
        
        for android_id in self.android_ids:
            if android_id == self.android_id:
                return True
        return False

    def get_rotation_angle(self) -> int:
        """
            获取手机旋转角度
            Get the rotation angle of the mobile phone

            return: 手机旋转的角度
            return: the angle at which the phone rotates.
        """
        response = self.SendData("getRotationAngle")
        return int(response)

    def hid_press(self, coordinate: tuple) -> bool:
        """
            按下
            press

            coordinate: x,y坐标
            return: True或者False

            coordinate x, y: abscissa
            return: True or False
        """
        self.angle = self.get_rotation_angle()
        return  self.win_driver.hid_press(self.android_id, self.angle, coordinate[0], coordinate[1]) 

    def hid_release(self) -> bool:
        """
            释放
            release

            return: True或者False
            return: True or False.
        """
        self.angle = self.get_rotation_angle()
        return self.win_driver.hid_release(self.android_id, self.angle) 

    def hid_move(self, coordinate: tuple, duration: float) -> bool:
        """
            移动
            move

            coordinate: x,y坐标
            duration: 移动时长,秒(移动时间内脚本需保持运行)
            return: True或者False

            coordinate: x,y coordinates
            duration: moving duration, seconds (the script should be kept running during the moving time)
            return: True or False
        """
        self.angle = self.get_rotation_angle()
        return self.win_driver.hid_move(self.android_id, self.angle, coordinate[0], coordinate[1], duration) 

    def hid_click(self, coordinate: tuple) -> bool:
        """
            单击
            click

            coordinate: x, y横坐标
            return: True或者False

            coordinate: x, y abscissa
            return: True or False
        """
        self.angle = self.get_rotation_angle()
        return self.win_driver.hid_click(self.android_id, self.angle, coordinate[0], coordinate[1]) 

    def hid_double_click(self, coordinate: tuple) -> bool:
        """
            双击
            double click

            coordinate: x,y横坐标
            return: True或者False

            coordinate: x,y abscissa
            return: True or False
        """
        self.angle = self.get_rotation_angle()
        return self.win_driver.hid_double_click(self.android_id, self.angle, coordinate[0], coordinate[1]) 

    def hid_long_click(self, coordinate: tuple, duration: float) -> bool:
        """
            长按
            Long press

            coordinate: x,y坐标
            duration: 按下时长,秒(按下时间内脚本需保持运行)
            return: True或者False

            coordinate: x,y coordinates
            duration: press duration, seconds (the script should be kept running during the press duration)
            return: True or False
        """
        self.angle = self.get_rotation_angle()
        return  self.win_driver.hid_long_click(self.android_id, self.angle, coordinate[0], coordinate[1], duration) 

    def hid_swipe(self, Startcoordinate: tuple,Endcoordinate: tuple, duration: float) -> bool:
        """
            滑动坐标
            Sliding coordinate

            Startcoordinate: x,y 起始坐标
            Endcoordinate: x,y 结束坐标
            duration: 滑动时长,秒(滑动时间内脚本需保持运行)
            return: True或者False

            Startcoordinate: x,y start coordinate
            Endcoordinate: x,y end coordinate
            duration: sliding duration, seconds (the script needs to be kept running during sliding time)
            return: True or False
        """
        self.angle = self.get_rotation_angle()
        return self.win_driver.hid_swipe(self.android_id, self.angle, Startcoordinate[0], Startcoordinate[1], Endcoordinate[0], Endcoordinate[1], duration) 

    def hid_gesture(self, gesture_path: list, duration: float) -> bool:
        """
            Hid手势
            Hid gesture

            gesture_path: 手势路径，由一系列坐标点组成
            duration: 手势执行时长, 单位秒(执行时间内脚本需保持运行)
            return: True或者False

            gesture_path: gesture path, which consists of a series of coordinate points
            duration: the duration of gesture execution, in seconds (the script should be kept running during the execution time)
            return: True or False
        """
        self.angle = self.get_rotation_angle()
        return self.win_driver.hid_gesture(self.android_id, self.angle, gesture_path, duration) 

    def hid_back(self) -> bool:
        """
            返回
            return
        """

        return self.win_driver.hid_back(self.android_id) 

    def hid_home(self) -> bool:
        """
            返回桌面
            home
        """
        return self.win_driver.hid_home(self.android_id) 

    def hid_recents(self) -> bool:
        """
            最近应用列表
            List of recent applications
        """
        return self.win_driver.hid_recents(self.android_id) 


import re
from ast import literal_eval


class OcrOperation:
    """
        OCR 操作
        OCR operation
    """

    # def init_ocr_server(self, ip: str, use_angle_model: bool = False, enable_gpu: bool = False, enable_tensorrt: bool = False) -> bool:
    #     """
    #         初始化 OCR 服务
    #         Initialize OCR service

    #         ip: ocr服务器IP，固定端口9527。当参数值为 "127.0.0.1"时 为本地DLL模式
    #         use_angle_model: 支持图像旋转
    #         enable_gpu: 启动GPU 模式。GPU 模式需要电脑安装 NVIDIA 驱动，并且到群文件下载对应 cuda 版本
    #         enable_tensorrt: 启动加速，仅 enable_gpu = True 时有效。图片太大可能会导致GPU内存不足。
    #         return: 总是返回true

    #         ip: ocr server IP, fixed port 9527. When the parameter value is "127.0.0.1", it is local DLL mode
    #         use_angle_model: supports image rotation
    #         enable_gpu: starts GPU mode. GPU mode requires the computer to install NVIDIA driver and download the corresponding cuda version from the group file
    #         enable_tensorrt: Enables acceleration, which is only valid when enable_gpu = True. Too large a picture may lead to insufficient GPU memory
    #         return: always returns true
    #     """
    #     return "true" in self.SendData("initOcr", ip, use_angle_model, enable_gpu, enable_tensorrt)

    def ocr_server_by_file(self, ocr_server_id: str, image_path: str, region: tuple = (0, 0, 0, 0), algorithm: tuple = (0, 0, 0)) -> list:
        """
            OCR 服务，通过 OCR 识别图片路径中的文字
            OCR service, which recognizes the characters in the image path through OCR

            ocr_server_id: ocr服务端IP，端口固定为9527
            image_path: 图片路径
            region: (左上角x点, 左上角y点, 右下角 x点, 右下角 y点)
            algorithm: (二值化算法类型, 阈值, 最大值)
            return: 失败返回null，成功返回列表形式的识别结果: [[[[317, 4], [348, 4], [348, 22], [317, 22]]]]

            ocr_server_id: the IP of the ocr server, and the port is fixed at 9527
            image_path: image path
            region: (point X in the upper left corner, point Y in the upper left corner, point X in the lower right corner, point Y in the lower right corner)
            algorithm: (Binary algorithm type, threshold, maximum)
            return: failed to return null, and successfully returned the recognition results in list form: [[[[317,4], [348,4], [348,22], [317,22]]]
        """
        algorithm_type, threshold, max_val = algorithm
        if algorithm_type in (5, 6):
            threshold = 127
            max_val = 255

        response = self.SendData("ocrByFile", ocr_server_id, image_path, *region, algorithm_type, threshold, max_val)
        if response == "null" or response == "":
            return []
        return literal_eval(response)

    def ocr_server_by_hwnd(self, ocr_server_id: str, hwnd: str, region: tuple = (0, 0, 0, 0), algorithm: tuple = (0, 0, 0), mode: bool = False) -> list:
        """
            OCR 服务，通过 OCR 识别窗口句柄中的文字
            OCR service, which recognizes the characters in the window handle through OCR
            
            ocr_server_id: ocr服务端IP，端口固定为9527
            hwnd: Window 句柄
            region: (左上角x点, 左上角y点, 右下角 x点, 右下角 y点)
            algorithm: (二值化算法类型, 阈值, 最大值)
            mode: 操作模式，后台 true，前台 false。默认前台操作   
            return: 失败返回null，成功返回列表形式的识别结果: [[[[317, 4], [348, 4], [348, 22], [317, 22]]]]

            ocr_server_id: the IP of the ocr server, and the port is fixed at 9527
            image_path: image path
            region: (point X in the upper left corner, point Y in the upper left corner, point X in the lower right corner, point Y in the lower right corner)
            algorithm: (Binary algorithm type, threshold, maximum)
            mode: Operation mode, background true, foreground false. Default foreground operation
            return: failed to return null, and successfully returned the recognition results in list form: [[[[317,4], [348,4], [348,22], [317,22]]]
        """
        algorithm_type, threshold, max_val = algorithm
        if algorithm_type in (5, 6):
            threshold = 127
            max_val = 255

        response = self.SendData("ocrByHwnd", ocr_server_id, hwnd, *region, algorithm_type, threshold, max_val, mode)
        if response == "null" or response == "":
            return []
        return literal_eval(response)

    def get_text(self, ocr_server_id: str, hwnd_or_image_path: str, region: tuple = (0, 0, 0, 0), algorithm: tuple = (0, 0, 0), mode: bool = False) -> list:
        """
            通过 OCR 识别窗口/图片中的文字，
            Recognize the characters in the window/picture through OCR, and return to the text list

            ocr_server_id: ocr服务端IP，端口固定为9527
            hwnd_or_image_path: 窗口句柄或者图片路径；
            region: 识别区域，默认全屏；
            threshold二值化图片, thresholdType算法类型：
                                                      0   THRESH_BINARY算法，当前点值大于阈值thresh时，取最大值maxva，否则设置为0
                                                      1   THRESH_BINARY_INV算法，当前点值大于阈值thresh时，设置为0，否则设置为最大值maxva
                                                      2   THRESH_TOZERO算法，当前点值大于阈值thresh时，不改变，否则设置为0
                                                      3   THRESH_TOZERO_INV算法，当前点值大于阈值thresh时，设置为0，否则不改变
                                                      4   THRESH_TRUNC算法，当前点值大于阈值thresh时，设置为阈值thresh，否则不改变
                                                      5   ADAPTIVE_THRESH_MEAN_C算法，自适应阈值
                                                      6   ADAPTIVE_THRESH_GAUSSIAN_C算法，自适应阈值
                                thresh阈值，maxval最大值，threshold默认保存原图。thresh和maxval同为255时灰度处理
            mode: 操作模式，后台 True，前台 False, 默认前台操作；
            return: 返回文字列表

            ocr_server_id: the IP of the ocr server, and the port is fixed at 9527
            hwnd_or_image_path: window handle or image path
            region: recognition region, full screen by default
            algorithm: the algorithm and parameters used to process the picture/screen, and the original picture is saved by default
            threshold binary image, thresholdType algorithm type:
                0 THRESH_BINARY algorithm, when the current point value is greater than the threshold thresh, take the maximum maxva, otherwise set to 0.
                1 THRESH_BINARY_INV algorithm, when the current point value is greater than the threshold thresh, set it to 0, otherwise set it to the maximum maxva.
                2 THRESH_TOZERO algorithm, when the current point value is greater than the threshold thresh, it will not change, otherwise it will be set to 0.
                3 THRESH_TOZERO_INV algorithm, when the current point value is greater than the threshold thresh, it is set to 0, otherwise it will not change.
                4 THRESH_TRUNC algorithm, when the current point value is greater than the threshold thresh, set it as the threshold thresh, otherwise it will not change.
                5 ADAPTIVE_THRESH_MEAN_C algorithm, adaptive threshold
                6 adaptive _ threshold _ Gaussian _ c algorithm, adaptive threshold.
                Threshold threshold, maxval maximum, threshold saves the original image by default. Gray processing when thresh and maxval are both 255.
            mode: operation mode, background True, foreground False, default foreground operation
            return: returns the text list
        """
        if hwnd_or_image_path.isdigit():
            text_info_list = self.ocr_server_by_hwnd(ocr_server_id, hwnd_or_image_path, region, algorithm, mode)
        else:
            text_info_list = self.ocr_server_by_file(ocr_server_id, hwnd_or_image_path, region, algorithm)

        return text_info_list

    def find_text(self, ocr_server_id: str, hwnd_or_image_path: str, text: str, region: tuple = (0, 0, 0, 0), algorithm: tuple = (0, 0, 0), mode: bool = False) -> list:
        """
            通过 OCR 识别窗口/图片中的文字
            Recognize the characters in the window/picture through OCR, and return the coordinate list

            ocr_server_id: ocr服务端IP，端口固定为9527
            hwnd_or_image_path: 句柄或者图片路径
            text: 要查找的文字
            region: 识别区域，默认全屏
            algorithm: 处理图片/屏幕所用算法和参数，默认保存原图
            threshold二值化图片, thresholdType算法类型：
                                                      0   THRESH_BINARY算法，当前点值大于阈值thresh时，取最大值maxva，否则设置为0
                                                      1   THRESH_BINARY_INV算法，当前点值大于阈值thresh时，设置为0，否则设置为最大值maxva
                                                      2   THRESH_TOZERO算法，当前点值大于阈值thresh时，不改变，否则设置为0
                                                      3   THRESH_TOZERO_INV算法，当前点值大于阈值thresh时，设置为0，否则不改变
                                                      4   THRESH_TRUNC算法，当前点值大于阈值thresh时，设置为阈值thresh，否则不改变
                                                      5   ADAPTIVE_THRESH_MEAN_C算法，自适应阈值
                                                      6   ADAPTIVE_THRESH_GAUSSIAN_C算法，自适应阈值
                                thresh阈值，maxval最大值，threshold默认保存原图。thresh和maxval同为255时灰度处理
            mode: 操作模式，后台 true，前台 false, 默认前台操作
            return: 文字坐标列表

            ocr_server_id: the IP of the ocr server, and the port is fixed at 9527
            hwnd_or_image_path: handle or image path
            text: the text to find
            region: recognition region, full screen by default
            algorithm: the algorithm and parameters used to process the picture/screen, and the original picture is saved by default
            threshold binary image, thresholdType algorithm type:
                0 THRESH_BINARY algorithm, when the current point value is greater than the threshold thresh, take the maximum maxva, otherwise set to 0.
                1 THRESH_BINARY_INV algorithm, when the current point value is greater than the threshold thresh, set it to 0, otherwise set it to the maximum maxva.
                2 THRESH_TOZERO algorithm, when the current point value is greater than the threshold thresh, it will not change, otherwise it will be set to 0.
                3 THRESH_TOZERO_INV algorithm, when the current point value is greater than the threshold thresh, it is set to 0, otherwise it will not change.
                4 THRESH_TRUNC algorithm, when the current point value is greater than the threshold thresh, set it as the threshold thresh, otherwise it will not change.
                5 ADAPTIVE_THRESH_MEAN_C algorithm, adaptive threshold
                6 adaptive _ threshold _ Gaussian _ c algorithm, adaptive threshold.
                Threshold threshold, maxval maximum, threshold saves the original image by default. Gray processing when thresh and maxval are both 255.
            mode: operation mode, background true, foreground false, default foreground operation
            return: list of text coordinates
        """

        if hwnd_or_image_path.isdigit():
            text_info_list = self.ocr_server_by_hwnd(ocr_server_id, hwnd_or_image_path, region, algorithm, mode)
        else:
            text_info_list = self.ocr_server_by_file(ocr_server_id, hwnd_or_image_path, region, algorithm)

        text_points = []
        for item in text_info_list:
            full_text = item["text"]
            if text in full_text:
                x1, y1, x2, y2 = item["box"]
                
                # 文本区域尺寸
                width = x2 - x1
                height = y2 - y1
                
                # 跳过空文本或无效 box
                if len(full_text) == 0 or width <= 0 or height <= 0:
                    continue
                
                # 等宽假设：每个字符宽度
                single_char_width = width / len(full_text)
                
                # 查找子串首次出现位置
                pos = full_text.find(text)
                if pos == -1:
                    continue  # 理论不会发生
                
                # 子串中心相对于 box 左上角的偏移
                sub_width = len(text) * single_char_width
                offset_x = pos * single_char_width + sub_width / 2
                offset_y = height / 2
                
                # 绝对坐标（加上 region 偏移）
                point_x = region[0] + x1 + offset_x
                point_y = region[1] + y1 + offset_y
                
                text_points.append((float(point_x), float(point_y)))
        
        return text_points


from setuptools import setup, find_packages


packages = find_packages()
install_requires = [
    "loguru",
    "psutil",
    "Pillow",
    "requests",
    "pymysql",
    "websockets",
    "asyncio"
]

setup(
    name="pyaibote", 
    version="2.0.2", 
    author="Riven", 
    author_email="pyaibote@163.com", 
    description="A pure code RPA office automation framework, which supports Android, Browser and Windows", 
    long_description="Support Android native APP and H5 interface elements and color positioning. The speed of element location is 10 times that of Appium framework, and the color location of 2340*1080 image only takes 50 milliseconds, It supports the positioning of window interface elements and colors developed by Windows applications,. NET, WinForm, WPF, QT, Java (GUI libraries such JAVA(Swing and AWT) and Electron. The exclusive xpath algorithm is concise and rapid, and the positioning speed of elements/colors is 3 times and 20 times that of visual RPA, respectively, All browsers and applications that support the chromium kernel. A web automation framework developed by C/C++ language based on browser kernel protocol. Ten times faster than Selenium", # 加载read_me的内容
    long_description_content_type="text/markdown", 
    url="https://www.pyaibote.com",  
    packages=packages, 
    package_data={"": ['*.py']},
    install_requires = install_requires,
    python_requires = ">=3.5,<4.0"
)


import re


class UrlRequest:
    """
        URL请求
        URL request
    """

    def url_request(self, url: str = "https://www.baidu.com", requestType: str = "GET", headers: str = "null", postData: str = "null") -> str:
        """
            获取请求地址html数据
            Get that html data of the request address

            url: 请求的地址 http://www.ai-bot.net
            requestType: 请求类型，GET或者POST
            headers: 可选参数，请求头
            postData: 可选参数，用作POST 提交的数据
            return: {Promise.<string>} 返回请求数据内容

            url: The requested address is http://www.ai-bot.net
            requestType: request type, GET or POST
            headers: optional parameter, request header
            postData: optional parameter, used as data submitted by post
            return: {Promise.<string>} returns the requested data content
        """
        response = self.SendData("urlRequest", url, requestType, headers, postData)
        if response == "null":
            return None
        return response

    def download_file(self, url: str, savePath: str) -> bool:
        """
            下载下载网络文件到手机
            Download download network files to your mobile phone

            url: 请求网络文件的地址
            savePath: 文件保存到手机哪个位置(默认手机根目录)
            return: Ture 或者 False

            url: the address of the requested network file.
            savePath: where the file is saved to the mobile phone (default mobile phone root directory)
            return: Ture or False
        """

        if not savePath.startswith("/storage/emulated/0/"):
            savePath = "/storage/emulated/0/" + savePath
            
        return "true" in self.SendData("downloadFile", url, savePath) 






class KeymouseOperation:
    """
        鼠标键盘操作
        Mouse and keyboard operation
    """

    def click_mouse_by_element(self, xpath: str, typ: int) -> bool:
        """
            根据元素位置点击鼠标(元素中心点)

            xpath: 元素 xpath 路径
            typ: 点击类型，单击左键:1 单击右键:2 按下左键:3 弹起左键:4 按下右键:5 弹起右键:6 双击左键:7
            return: 布尔值

            xpath: Element xpath path
            typ: Click type, click left key: 1 Right click: 2 Press left key: 3 Pop up left key: 4 Press right key: 5 Pop up right key: 6 Double click left key: 7
            return: bool
        """
        return "true" in self.SendData("clickMouseByXpath", xpath, typ)

    def move_to_element(self, xpath: str) -> bool:
        """
            移动鼠标到元素位置(元素中心点)
            Move the mouse to the element position (element center point)

            xpath: 元素 xpath 路径
            return: 布尔值

            xpath: Element xpath path
            return: bool
        """
        return "true" in self.SendData("moveMouseByXpath", xpath)

    def scroll_mouse_by_element(self, xpath: str, offset_x: float, offset_y: float) -> bool:
        """
            根据元素位置滚动鼠标
            Scroll the mouse according to the element position.

            xpath: 元素路径
            offset_x: 水平滚动条移动的距离
            offset_y: 垂直滚动条移动的距离
            return: 布尔值

            xpath: xpath path
            offset_x: The distance the horizontal scroll bar moves
            offset_y: Distance the vertical scroll bar moves.
            return: bool
        """
        return "true" in self.SendData("wheelMouseByXpath", xpath, offset_x, offset_y)

    def click_mouse(self, point: tuple, typ: int) -> bool:
        """
            点击鼠标
            click the mouse

            point: 坐标点
            typ: 点击类型，单击左键:1 单击右键:2 按下左键:3 弹起左键:4 按下右键:5 弹起右键:6 双击左键:7
            return: 布尔值

            point: coordinate point
            typ: Click type, click left key: 1 Click right key: 2 Press left key: 3 Pop up left key: 4 Press right key: 5 Pop up right key: 6 Double click left key: 7
            return: Boolean value
        """
        return "true" in self.SendData("clickMouse", point[0], point[1], typ) 

    def move_mouse(self, point: tuple) -> bool:
        """
            移动鼠标
            Move the mouse

            point: 坐标点
            return: 布尔值

            point: coordinate point
            return: Boolean value
        """
        return "true" in self.SendData("moveMouse", point[0], point[1])

    def scroll_mouse(self, point: tuple = (0,0), offset_x: float = 0, offset_y: float = 100) -> bool:
        """
            滚动鼠标
            Scroll mouse

            point 鼠标x, y坐标位置
            offset_x: 水平滚动条移动的距离
            offset_y: 垂直滚动条移动的距离
            return: 布尔值

            point mouse x, y coordinate position
            offset_x: the distance that the horizontal scroll bar moves
            offset_y: the distance that the vertical scroll bar moves
            return: Boolean value
        """
        return "true" in self.SendData("wheelMouse", offset_x, offset_y, point[0], point[1])





"""
    PyAibote Windows自动化框架，由WindowsDriver.exe客户端驱动程序连接脚本服务端，采用TCP协议传输命令。
    全国领先xpath元素定位算法，一键拾取控件元素、图色 坐标等属性。与安卓端代码统一书写格式，降低用户学习成本。
    独家图色定位算法，50毫秒以内的速度遍历查找1920*1080分辨率的设备

    PyAibote Windows automation framework, with WindowsDriver.exe client driver connected to script server and TCP protocol used to transmit commands.
    The leading xpath element positioning algorithm in China can pick up control elements, color coordinates and other attributes with one click. 
    The writing format is unified with the Android code, which reduces the learning cost of users.
    Exclusive color positioning algorithm, searching for equipment with resolution of 1920*1080 at a speed of less than 50 milliseconds

"""


import os,sys
import time
import subprocess
from abc import ABC, abstractmethod
import socketserver,socket
from .Tool import  *
from .WindowsBotModel import  *
import threading



class WinBotMain(
        ABC,
        LoggerRecord,
        ThreadingTCPServer, 
        WinLoadWait,
        SendClientData,
        DrivingOperation,
        WindowOperation,
        KeymouseOperation,
        ColorOperation,
        OcrOperation,
        YoloOperation,
        ElementOperation,
        SystemOperation,
        OtherOperations,
        ExcelOperation,
        VoiceService,
        DigitalHumanOperation,
        VerificationCodeOperation,
        WinHidCorrelation,
        ChatGenerative,
        DataBaseHandle,
        Sqlite3DataBaseHandle,
        WebSocketServerUse
    ):

    def __init__(self,*args):
        if len(args) ==1:
            address_info = socket.getaddrinfo(None, args[0], socket.AF_INET, socket.SOCK_STREAM)[0]
            family, socket_type, proto, _, socket_address = address_info
            server = socket.socket(family, socket_type, proto)
            server.bind(socket_address)
            server.listen(1)
            print("WindowsBot Service started successfully ...")
            self.request, self.client_address = server.accept()
            print("WindowsBot Client link succeeded")
        else:
            super().__init__(*args)


            
    @classmethod
    def _build(self, listen_ip:str, listen_port: int, Debug: bool = True) -> object:
        """
            使用安卓hid模式时需要启动windows驱动
            Windows driver needs to be started when using Android hid mode.

            listen_port: 脚本监听的端口
            Debug: 脚本是否部署在本地
            return: windows类对象

            listen_port: the port on which the script listens.
            Debug: Is the script deployed locally?
            return: windows class object
        """
        if listen_port < 0 or listen_port > 65535:
            raise OSError("`listen_port` must be in 0-65535.")

        if Debug:
            try:
                os.popen(f"WindowsDriver.exe {listen_ip} {listen_port}")
                print("Debug Model Start WinDriver ...")
            except FileNotFoundError as e:
                err_msg = "\nStart local WinDriver.exe fail Exception elimination step：\n1. Check WebDriver.exe Path；\n2. WebDriver.exe Add to system environment variable?"
                self.error(f"{err_msg}: " + str(e))

        return WinBotMain(listen_port)

    # @abstractmethod
    def script_main(self):
        pass
    
    def handle(self):
        self.info(f"<-<- Client connection at {self.client_address[0]}: {self.client_address[1]}")
        self.debug(f"<-<- Client connection at {self.client_address[0]}: {self.client_address[1]}")
        self.script_main()

    @classmethod
    def execute(self, IP: str, Port: int, Debug: bool = True, Qt = None, WebsocketSwitch = False, WebsocketPort = 8888):
        try:
            if Port < 0 or Port > 65535:
                raise OSError("`listen_port` must be in 0-65535.")
            
            if Qt:
                self.Qt = Qt

            if Debug:
                Driver.WindowsDriverStart(IP, Port)

            if WebsocketSwitch:
                server = WebSocketServerThread(IP, WebsocketPort, self.Websocket_Log_Level)
                server.start()

            ThreadingTCPServer.StartThreadingTCPServer(self, IP, Port)
        except KeyboardInterrupt as e:
            sys.exit(self)

    @classmethod
    def StopSrver(self):
        try:
            self.server.shutdown()  # 停止接受新的连接
            self.server.socket.close()  # 关闭服务器套接字
        except Exception as e:
            pass



















import time
import copy


class WebLoadWait:
    """
        隐式等待和显示等待
        Implicit waiting and explicit waiting
    """

    Implicit_Waiting = 0
    Implicit_Waiting_Frequency = 0
    Implicit_Waiting_Throwing = False
    Show_Waiting = 0
    Show_Waiting_Frequency = 0
    Show_Waiting_Throwing = False

    def _Send(self,data):
        self.debug(rf"->>> {data}")

        self.request.sendall(data)
        response = self.request.recv(87654)
        if response == b"":
            self.request.close()
            raise ConnectionAbortedError(f"{self.client_address[0]}:{self.client_address[1]} Client disconnects")
        data_length, data = response.split(b"/", 1)
        while int(data_length) > len(data):
            data += self.request.recv(87654)

        response = data.decode('UTF-8')
        if len(response) > 10000:
            self.debug(rf"<-<- {response[:100]}......")
        else:
            self.debug(rf"<-<- {response}")
        if response == b"":
            self.request.close()
            raise ConnectionAbortedError(f"{self.client_address[0]}:{self.client_address[1]} Client disconnects")
        return response

    def StarLoadWait(self, data):
        response = self._Send(data)
        start_time = time.time()
        if 'false' in response or 'webdriver error' in response:
            while time.time() - start_time < self.Implicit_Waiting:
                try:
                    self.debug(rf"---- Cannot find element. Retrying....  No retry in {self.Implicit_Waiting}s   {round(time.time() - start_time, 2)}s")
                    response = self._Send(data)
                    if 'false' not in response and 'webdriver error' not in response:
                        break
                    if "domain" in response or "path" in response or "secure" in response or "httpOnly" in response:
                        break
                    time.sleep(self.Implicit_Waiting_Frequency)
                except Exception as e:
                    time.sleep(self.Implicit_Waiting_Frequency)
            if self.Implicit_Waiting_Throwing:
                if 'false' in response or 'webdriver error' in response:
                    self.request.close()
                    raise "Unable to find the specified element"
        return response

    def StartShowWait(self, Time: int, RotationTime: int, ThrowException: bool):
        """
            进行隐式等待和显示等待模式的切换
            Switch between implicit waiting and display waiting modes.
        """
        self.debug(rf"---- Enter the display waiting mode waiting time：{Time}s")
        self.Show_Waiting = copy.deepcopy(self.Implicit_Waiting)
        self.Show_Waiting_Frequency = copy.deepcopy(self.Implicit_Waiting_Frequency)
        self.Show_Waiting_Throwing = copy.deepcopy(self.Implicit_Waiting_Throwing)

        self.Implicit_Waiting = Time
        self.Implicit_Waiting_Frequency = RotationTime
        self.Implicit_Waiting_Throwing = ThrowException

    def EndShowWait(self):
        """
            进行隐式等待和显示等待模式的切换
            Switch between implicit waiting and display waiting modes.
        """
        self.debug(rf"---- Finish the display waiting mode")
        self.Implicit_Waiting = self.Show_Waiting
        self.Implicit_Waiting_Frequency = self.Show_Waiting_Frequency
        self.Implicit_Waiting_Throwing = self.Show_Waiting_Throwing


from .AndroidLoadWait import  AndroidLoadWait
from .CoordinateOperation import  CoordinateOperation
from .ElementOperation import  ElementOperation
from .EquipmentOperation import  EquipmentOperation
from .ScreenProjectionOperation import  ScreenProjectionOperation
from .FileTransfer import  FileTransfer
from .Control import  Control
from .OcrCorrelation import  OcrCorrelation
from .YoloService import  YoloService
from .UrlRequest import  UrlRequest
from .ColorFindingOperation import  ColorFindingOperation
from .MapFindingOperation import  MapFindingOperation
from .ScreenshotOperation import  ScreenshotOperation
from .AndroidHidCorrelation import  AndroidHidCorrelation
from ..CommonUse.ChatGenerative import ChatGenerative
from .VerificationCodeOperation import  VerificationCodeOperation

__all__ = ["AndroidLoadWait","CoordinateOperation","ElementOperation","EquipmentOperation",
            "ScreenProjectionOperation","FileTransfer","Control","OcrCorrelation","YoloService",
            "UrlRequest","ColorFindingOperation","MapFindingOperation","ScreenshotOperation",
            "AndroidHidCorrelation","ChatGenerative","VerificationCodeOperation"]




class IframeOperation:
    """
        iframe 操作
        IframeOperation
    """

    def switch_to_frame(self, xpath) -> bool:
        """
            切换到指定 frame
            Switch to the specified frame

            xpath: xpath 路径
            return: 布尔值

            xpath: xpath path
            return: bool
        """
        return "true" in self.SendData("switchFrame", xpath)

    def switch_to_main_frame(self) -> bool:
        """
            切回主 frame
            Switch back to the main frame

            return: 布尔值
            return: bool

        """
        return "true" in self.SendData("switchMainFrame")



import json,requests


class YoloService:
    """
        Yolo 目标检测
        Yolo target detection
    """

    # def init_yolo_server(self, ip: str, model_path: str , classes_path:str) -> bool:
    #     """
    #         初始化 yolo 服务
    #         Initialize yolo service

    #         ip: yolo 服务 IP 或域名，端口固定9528
    #         model_path: 模型路径比如：  "d:/yolov8n.onnx"
    #         classes_path 种类路径，CPU模式需要此参数比如： "d:/classes.txt"
    #         return: True 或者 False

    #         ip: yolo service IP or domain name, with fixed port 9528
    #         model_path: model path, such as "d:/yolov8n.onnx"
    #         classes_path path, which is required for CPU mode, such as "d:/classes.txt"
    #         return: True or False
    #     """
    #     return "true" in self.SendData("initYolo", ip, model_path, classes_path) 

    def yolo(self, ocr_server_id: str, region: tuple = (0,0,0,0), scale: float = 1.0) -> list:
        """
            yolo 目标检测
            Yolo target detection

            ocr_server_id: yolo服务端IP，端口固定为9528
            scale: 图片缩放率, 默认为 1.0 原大小。大于 1.0 放大，小于 1.0 缩小，不能为负数。
            region: 识别区域，默认全屏
            return: 失败返回[]，成功返回数组形式的识别结果， 0~3目标矩形位置  4目标类别  5置信度

            ocr_server_id: Yolo server IP, the port is fixed at 9528
            scale: the image scaling rate, which defaults to 1.0 original size. Magnification greater than 1.0 and reduction less than 1.0 cannot be negative
            region: Recognition area, default full screen
            return: failed to return [], successfully returned the recognition result in the form of array, 0~3 target rectangle position 4 target category 5 confidence
        """
        response = self.SendData("yolo", ocr_server_id, *region, scale)
        if response == "null" or response == "":
            return []
        return json.loads(response)

    def yolo_tool_by_file(self, tool_server_ip: str, tool_server_port: str, file_path: str, is_get_class_name: str = '0') -> dict:
        """
            yolo综合工具 根据图片进行目标检测
            Yolo integrated tools detect targets according to pictures

            tool_server_ip: 工具部署电脑的IP地址 本机部署就是 127.0.0.1, 公网写公网IP
            tool_server_port： 需要和工具上面填写的端口号一致
            file_path: 图片路径
            region: (左上角x点, 左上角y点, 右下角 x点, 右下角 y点)
            is_get_class_name： 获取pt模型中所有训练的类名 0：不获取  1：获取
            return:  失败返回空{}，成功返回数组形式的识别结果。 0~3目标矩形位置  4目标类别  5置信度

            tool_server_ip: the IP address of the tool deployment computer. The local deployment is 127.0.0.1, and the public network writes the public network IP.
            tool_server_port: it needs to be consistent with the port number filled in the tool.
            file_path: image path.
            region: (point X in the upper left corner, point Y in the upper left corner, point X in the lower right corner, point Y in the lower right corner)
            is_get_class_name: Get the class names of all trainings in pt model 0: Don't get 1: Get.
            return: failed to return an empty {}, and successfully returned the recognition result in the form of an array. 0~3 Target Rectangular Position 4 Target Category 5 Confidence
        """

        url = f'http://{tool_server_ip}:{tool_server_port}/pt_model_predict'

        with open(file_path, 'rb') as f:
            image_content = f.read()

        files = {'file': image_content}
        data = {'get_class_name': is_get_class_name}

        response = requests.post(url, files=files, data=data)
        if response.status_code == 200:
            data_json = json.loads(response.text)
            return data_json
        else:
            return {}


"""
	Mysql数据库操作
	Mysql database operation
"""
import pymysql
import os,sys,time
from .WriteReadFileFunc import  WriteReadFile
import traceback

class DataBaseHandle(WriteReadFile):
	def init_mysql(self)  -> bool:
		self.Path = f"{os.getcwd()}/ConfigFile.json"
		if not os.path.exists(self.Path):
			with open(self.Path,"w",encoding='UTF-8') as w:
				w.write('')

		self.USERNAME = WriteReadFile.ReadJsonFile(self,self.Path,"LinkDatabaseInfo","UserName")
		self.PASSWORD = WriteReadFile.ReadJsonFile(self,self.Path,"LinkDatabaseInfo","PassWord")
		self.DB = WriteReadFile.ReadJsonFile(self,self.Path,"LinkDatabaseInfo","DBName")
		self.host= WriteReadFile.ReadJsonFile(self,self.Path,"LinkDatabaseInfo","Host")
		self.port = WriteReadFile.ReadJsonFile(self,self.Path,"LinkDatabaseInfo","Port")
		
		# 打开数据库连接
		try:
			self.db = pymysql.connect(host=self.host,user=self.USERNAME,password=self.PASSWORD,database=self.DB,port=self.port,cursorclass=pymysql.cursors.DictCursor)
			return True
		except Exception as e:
			print("""请在ConfigFile.json 中配置你的数据库信息示例：
			 		{
						"LinkDatabaseInfo":{
							"UserName":"root",
							"PassWord":"12345678",
							"DBName":"pyaibote",
							"Host":"192.168.31.1",
							"Port":3306
						}
					}
				""")
			WriteReadFile.Custom_Write_logger(self,"",f"{os.getcwd()}/AiBotRunLOG/{time.strftime(r'%Y-%m-%d',time.localtime(time.time()))}/","SysModeError.log",False,"error",f"{traceback.format_exc()}")
			return False

	def Checkconnect(self)  -> None:
		try:
			self.db.ping()
		except Exception as e:
			WriteReadFile.Custom_Write_logger(self,"",f"{os.getcwd()}/AiBotRunLOG/{time.strftime(r'%Y-%m-%d',time.localtime(time.time()))}/","SysModeError.log",False,"error",f"{traceback.format_exc()}")


	def insert_mysql(self,sql) -> bool:
		"""
			新增一条MySql数据库数据
			Add a MySql database data

			sql: sql语句
			return: 成功返回True, 失败返回False

			sql: sql statement
			return: Returns True on success and False on failure
		"""
		try:
			self.Checkconnect()
			cursor = self.db.cursor()
			result = cursor.execute(sql)
			self.db.commit()
			if result ==1:
				return True
			else:
				return False
		except Exception as e:
			WriteReadFile.Custom_Write_logger(self,"",f"{os.getcwd()}/AiBotRunLOG/{time.strftime(r'%Y-%m-%d',time.localtime(time.time()))}/","SysModeError.log",False,"error",f"{traceback.format_exc()}")
			return False


	def insertmany_mysql(self,sql,Data)  -> bool:
		"""
			新增多条MySql数据库数据
			Add multiple MySql database data

			sql: sql语句
			Data: 数据列表
			return: 成功返回True, 失败返回False

			sql: sql statement
			Data: data list
			return: Returns True on success and False on failure
		"""
		try:
			self.Checkconnect()
			cursor = self.db.cursor()
			cursor.executemany(sql,Data)
			self.db.commit()
			return True
		except Exception as e:
			WriteReadFile.Custom_Write_logger(self,"",f"{os.getcwd()}/AiBotRunLOG/{time.strftime(r'%Y-%m-%d',time.localtime(time.time()))}/","SysModeError.log",False,"error",f"{traceback.format_exc()}")
			return False
			
	def delete_mysql(self,sql) -> bool:
		"""
			删除MySql数据库数据
			Delete multiple MySql database data

			sql: sql语句
			return: 成功返回True, 失败返回False

			sql: sql statement
			return: True on success and False on failure
		"""

		try:
			self.Checkconnect()
			cursor = self.db.cursor()
			cursor.execute(sql)
			self.db.commit()
			return True
		except Exception as e:
			WriteReadFile.Custom_Write_logger(self,"",f"{os.getcwd()}/AiBotRunLOG/{time.strftime(r'%Y-%m-%d',time.localtime(time.time()))}/","SysModeError.log",False,"error",f"{traceback.format_exc()}")
			return False

	def update_mysql(self,sql)  -> bool:
		"""
			更新MySql数据库数据
			Update multiple MySql database data

			sql: sql语句
			return: 成功返回True, 失败返回False

			sql: sql statement
			return: True on success and False on failure
		"""

		try:
			self.Checkconnect()
			cursor = self.db.cursor()
			cursor.execute(sql)
			self.db.commit()
			return True
		except Exception as e:
			WriteReadFile.Custom_Write_logger(self,"",f"{os.getcwd()}/AiBotRunLOG/{time.strftime(r'%Y-%m-%d',time.localtime(time.time()))}/","SysModeError.log",False,"error",f"{traceback.format_exc()}")
			return False

	def select_mysql(self,sql)  -> dict:
		"""
			查询MySql数据库数据
			Query MySql database data

			sql: sql语句
			return: 成功返回字典数据，失败返回Error

			sql: sql statement
			return: dictionary data is returned successfully, and Error is returned if it fails
		"""

		try:
			self.Checkconnect()
			cursor = self.db.cursor()
			cursor.execute(sql)
			data=cursor.fetchall()
			return data

		except Exception as e:
			WriteReadFile.Custom_Write_logger(self,"",f"{os.getcwd()}/AiBotRunLOG/{time.strftime(r'%Y-%m-%d',time.localtime(time.time()))}/","SysModeError.log",False,"error",f"{traceback.format_exc()}")
			return False

			
	def close_mysql(self)  -> bool:
		try:
			if self.db:
				self.db.close()
			return True
		except Exception as e:
			WriteReadFile.Custom_Write_logger(self,"",f"{os.getcwd()}/AiBotRunLOG/{time.strftime(r'%Y-%m-%d',time.localtime(time.time()))}/","SysModeError.log",False,"error",f"{traceback.format_exc()}")
			return False




import sys

if sys.version_info < (3, 5):
    raise RuntimeError("最低支持 Python3.7 版本，请升级！")

from .WebBot import WebBotMain
from .WindowsBot import WinBotMain
from .AndroidBot import AndroidBotMain
from .DigitalHumanBot import HumanBotMain


__all__ = ["WebBotMain", "WinBotMain","AndroidBotMain","HumanBotMain"]




from .WinLoadWait import  WinLoadWait
from .DrivingOperation import  DrivingOperation
from .WindowOperation import  WindowOperation
from .KeymouseOperation import  KeymouseOperation
from .ColorOperation import  ColorOperation
from .OcrOperation import  OcrOperation
from .YoloOperation import  YoloOperation
from .ElementOperation import  ElementOperation
from .SystemOperation import  SystemOperation
from .OtherOperations import  OtherOperations
from .ExcelOperation import  ExcelOperation
from .VoiceService import  VoiceService
from .DigitalHumanOperation import  DigitalHumanOperation
from .VerificationCodeOperation import  VerificationCodeOperation
from .WinHidCorrelation import  WinHidCorrelation
from ..CommonUse.ChatGenerative import ChatGenerative

__all__ = ["WinLoadWait","DrivingOperation","WindowOperation","KeymouseOperation","ColorOperation","OcrOperation","YoloOperation",
            "ElementOperation","SystemOperation","OtherOperations","ExcelOperation","VoiceService","DigitalHumanOperation",
            "VerificationCodeOperation","WinHidCorrelation","ChatGenerative"] 


import re
import json,requests

class YoloOperation:
    """
        yolo目标检测
        Yolo target detection
    """

    # def init_yolo_server(self, ip: str, model_path: str = "d:/yolov8n.onnx", classesp_path: str = "d:/classes.txt") -> bool:
    #     """
    #         初始化 yolo 服务
    #         Yolo Operation

    #         ip: OCR 服务 IP 或域名，端口固定9528
    #         model_path: 模型路径
    #         classesp_path: 种类路径，CPU模式需要此参数
    #         return: 成功返回 True，失败返回False

    #         ip: OCR service IP or domain name, with fixed port 9528
    #         model_path: model path
    #         classesp_path: class path, which is required for CPU mode
    #         return: Returns True on success and False on failure
    #     """
    #     return "true" in self.SendData("initYolo", ip, model_path, classesp_path)

    def yolo_by_hwnd(self, ocr_server_id: str, hwnd: int, region: tuple = (0, 0, 0, 0), mode: bool = False) -> list:
        """
            yolo 根据窗口句柄目标检测
            Yolo target detection based on window handle

            ocr_server_id: yolo服务端IP，端口固定为9528
            hwnd: 窗口句柄
            region: (左上角x点, 左上角y点, 右下角 x点, 右下角 y点)
            mode: 操作模式，后台 True，前台 False。默认前台操作 
            return: 失败返回空[]，成功返回数组形式的识别结果。 0~3目标矩形位置  4目标类别  5置信度

            ocr_server_id: Yolo server IP, the port is fixed at 9528
            hwnd: Window handle
            region: (point X in the upper left corner, point Y in the upper left corner, point X in the lower right corner, point Y in the lower right corner)
            mode: Operation mode, background True, foreground False. Default foreground operation
            return: Failed to return empty [], successfully returned the recognition result in the form of array. 0~3 Target Rectangular Position 4 Target Category 5 Confidence
        """
        response = self.SendData("yoloByHwnd", ocr_server_id, hwnd, *region, mode)
        if response == "null" or response == "":
            return []
        return json.loads(response)

    def yolo_by_file(self, ocr_server_id: str, file_path: str, region: tuple = (0, 0, 0, 0)) -> list:
        """
            yolo 根据图片进行目标检测
            Yolo carries out target detection according to the picture.

            ocr_server_id: yolo服务端IP，端口固定为9528
            file_path: 图片路径
            region: (左上角x点, 左上角y点, 右下角 x点, 右下角 y点)
            return:  失败返回空[]，成功返回数组形式的识别结果。 0~3目标矩形位置  4目标类别  5置信度

            ocr_server_id: Yolo server IP, the port is fixed at 9528
            file_path: Picture path
            region: (point X in the upper left corner, point Y in the upper left corner, point X in the lower right corner, point Y in the lower right corner)
            return:  Failed to return empty [], successfully returned the recognition result in the form of array. 0~3 Target Rectangular Position 4 Target Category 5 Confidence
        """
        response = self.SendData("yoloByFile", ocr_server_id, file_path, *region)
        if response == "null" or response == "":
            return []
        return json.loads(response)

    def yolo_tool_by_file(self, tool_server_ip: str, tool_server_port: str, file_path: str, is_get_class_name: str = '0') -> dict:
        """
            yolo综合工具 根据图片进行目标检测
            Yolo integrated tools detect targets according to pictures

            tool_server_ip: 工具部署电脑的IP地址 本机部署就是 127.0.0.1, 公网写公网IP
            tool_server_port： 需要和工具上面填写的端口号一致
            file_path: 图片路径
            region: (左上角x点, 左上角y点, 右下角 x点, 右下角 y点)
            is_get_class_name： 获取pt模型中所有训练的类名 0：不获取  1：获取
            return:  失败返回空{}，成功返回数组形式的识别结果。 0~3目标矩形位置  4目标类别  5置信度

            tool_server_ip: the IP address of the tool deployment computer. The local deployment is 127.0.0.1, and the public network writes the public network IP.
            tool_server_port: it needs to be consistent with the port number filled in the tool.
            file_path: image path.
            region: (point X in the upper left corner, point Y in the upper left corner, point X in the lower right corner, point Y in the lower right corner)
            is_get_class_name: Get the class names of all trainings in pt model 0: Don't get 1: Get.
            return: failed to return an empty {}, and successfully returned the recognition result in the form of an array. 0~3 Target Rectangular Position 4 Target Category 5 Confidence
        """

        url = f'http://{tool_server_ip}:{tool_server_port}/pt_model_predict'

        with open(file_path, 'rb') as f:
            image_content = f.read()

        files = {'file': image_content}
        data = {'get_class_name': is_get_class_name}

        response = requests.post(url, files=files, data=data)
        if response.status_code == 200:
            data_json = json.loads(response.text)
            return data_json
        else:
            return {}


import requests
import json
import traceback 

class ChatGenerative:
    """
        对接第三方ChatGPT模型
        Docking third-party ChatGPT model
    """

    def wen_xin_bot(self,api_key:str, secret_key:str, content:str) -> dict:
        """
            文心一言ChatGPT模型
            ERNIE Bot ChatGPT model

            api_key: 文心一言签发的api_key, 获取地址：https://console.bce.baidu.com/
            secret_key ：文心一言签发的秘钥
            content: 你想问的问题
            return: 字典

            api_key: api_key issued by ERNIE Bot, obtained at https://console.bce.baidu.com/
            secret_key: the secret key issued by ERNIE Bot.
            content: the question you want to ask
            return: dictionary
        """

        def get_access_token():
            """
                使用 AK，SK 生成鉴权签名（Access Token）
                :return: access_token，或是None(如果错误)
            """
            url = "https://aip.baidubce.com/oauth/2.0/token"
            params = {"grant_type": "client_credentials", "client_id": api_key, "client_secret": secret_key}
            return str(requests.post(url, params=params).json().get("access_token"))


        try:
            url = "https://aip.baidubce.com/rpc/2.0/ai_custom/v1/wenxinworkshop/chat/completions?access_token=" + get_access_token()
            payload = json.dumps({
                "messages": [
                    {
                        "role": "user",
                        "content": content
                    }
                ],
                "disable_search": False,
                "enable_citation": False
            })
            headers = {
                'Content-Type': 'application/json'
            }
            
            response = requests.request("POST", url, headers=headers, data=payload)
            response = json.loads(response.text)
        except Exception as e:
            return {"err":traceback.format_exc()}
        return response

